!
Subroutine rttov_k( &
      & errorstatus,    &! out
      & nprofiles,      &! in
      & nchannels,      &! in
      & channels,       &! in
      & lprofiles,      &! in
      & addinterp,      &! in
      & profiles,       &! in
      & profiles_k,     &! inout  K mat on profile variables
      & coef,           &! in
      & coef_scatt_ir,  &! in
      & optp,           &! in
      & addsun,         &! in
      & addaerosl,      &! in
      & addclouds,      &! in
      & switchrad,      &! in
      & calcemis,       &! in
      & emissivity,     &! inout  direct model
      & emissivity_k,   &! inout  K mat on surface emissivity
      & transmission,   &! inout  K model
      & transmission_k, &! inout  K input
      & radiancedata,   &! inout  direct model   (input due to pointers alloc)
      & radiance_k,     &! inout OPTIONAL K radiances 
      & temp,           &
      & temp_k     ) 
  !
  ! Description:
  ! K matrix of rttov_direct
  ! to compute multi-channel level to space transmittances,
  ! top of atmosphere and level to space radiances and brightness
  ! temperatures and optionally surface emissivities, for many
  ! profiles in a single call, for satellite
  ! infrared or microwave sensors. The code requires a coefficient file
  ! for each sensor for which simulated radiances are requested.
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2002, EUMETSAT, All Rights Reserved.
  !
  ! Method: The methodology is described in the following:
  !
  ! Eyre J.R. and H.M. Woolf  1988 Transmittance of atmospheric gases
  ! in the microwave region: a fast model. Applied Optics 27  3244-3249
  !
  ! Eyre J.R. 1991 A fast radiative transfer model for satellite sounding
  ! systems.  ECMWF Research Dept. Tech. Memo. 176 (available from the
  ! librarian at ECMWF).
  !
  ! Saunders R.W., M. Matricardi and P. Brunel 1999 An Improved Fast Radiative
  ! Transfer Model for Assimilation of Satellite Radiance Observations.
  ! QJRMS, 125, 1407-1425.
  !
  ! Matricardi, M., F. Chevallier and S. Tjemkes 2001 An improved general
  ! fast radiative transfer model for the assimilation of radiance
  ! observations. ECMWF Research Dept. Tech. Memo. 345
  ! (available from the librarian at ECMWF).
  !
  ! Matricardi, M. 2003 RTIASI-4, a new version of the ECMWF fast radiative
  ! transfer model for the infrared atmospheric sounding interferometer.
  ! ECMWF Research Dept. Tech. Memo. 425 (available from the librarian at ECMWF)
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !  1.1   01/12/2002  New F90 code with structures (P Brunel A Smith)
  !  1.2   02/01/2003  More comments added (R Saunders)
  !  1.3   24/01/2003  Error return code by input profile (P Brunel)
  !                    Add WV Continuum and CO2 capability
  !  1.4   02/06/2004  Change tests on id_comp_lvl == 7 by tests on 
  !                    fmv_model_ver (P. Brunel)
  !  1.5   17/02/2005  Changed to allow calls from RTTOV_SCATT_K and 
  !                    RTTOV_CLD_K.   (A. Collard)
  !  1.6   24/08/2005  More changes so routine still works for clear 
  !                    sky RTTOV (R.Saunders)
  !  1.7   08/12/2005  Added surface humidity to lowest mean layer q (R Saunders) 
  !  1.8   13/03/2006  Marco Matricardi (ECMWF):
  !              --    IASI capability added.
  !              --    Variable trace gases added for IASI and AIRS.
  !              --    Solar radiation added for IASI and AIRS
  !              --    Altitude dependent local zenith angle added.
  !              --    Linear in tau approximation for RT equation added.
  !  1.9   08/02/2007  Removed polarisation index (Roger Saunders)
  !  1.10  11/10/2007  Profile allocation though rttov_alloc_prof
  !                    Move iaernum & iaertyp members to profile_aux P.Marguinaud
  !  1.11   16/01/2008 Facility to apply regression limits (N. Bormann)
  !  1.12   26/09/2008 Nullify temp pointers; this is for the NEC 
  !                    under -Chopt. P. Marguinaud
  !
  !  1.12   03/10/2008 Fix bug in checking addsolar (A. Geer)
  !  1.13   08/12/2008 Fixed bug in addcosmic for polarim (R Saunders)
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  ! A user guide and technical documentation is available at
  ! http://www.metoffice.com/research/interproj/nwpsaf/rtm/index.html
  !
  ! Declarations:
  ! Modules used:
  ! Imported Parameters:
  Use rttov_const, Only :   &
        & errorstatus_success ,&
        & errorstatus_warning ,&
        & errorstatus_fatal   ,&
        & max_optical_depth   ,&
        & sensor_id_mw        ,&
        & sensor_id_ir        ,&
        & sensor_id_po        ,&
        & sensor_id_hi        ,& 
        & ncldtyp             ,&
        & ngases_max          ,&
        & apply_reg_limits

  USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

  ! Imported Type Definitions:
  Use rttov_types, Only :              &
        & rttov_coef,                  &
        & rttov_coef_scatt_ir,         &
        & transmission_scatt_ir_type,  &
        & rttov_optpar_ir,             &
        & profile_Type,                &
        & geometry_Type,               &
        & predictors_Type,             &
        & profile_aux,                 &
        & opdp_path_type,              &
        & transmission_Type,           &
        & radiance_Type,               &
        & radiance_aux,                &
        & raytracing_type,             &
        & sunglint_type,               &
        & ircld_type,                  &
        & blob_type,                   & 
        & rttov_temp 

  Use rttov_global, Only: &
        & verbose_checkinput_warnings

  Use parkind1, Only : jpim     ,jprb
  Implicit None


  !subroutine arguments:
  Integer(Kind=jpim),        Intent(in)    :: nprofiles
  Integer(Kind=jpim),        Intent(in)    :: nchannels
  Integer(Kind=jpim),        Intent(in)    :: channels(nchannels)
  Integer(Kind=jpim),        Intent(in)    :: lprofiles(nchannels)
  Logical,                   Intent(in)    :: addsun
  Logical,                   Intent(in)    :: addaerosl
  Logical,                   Intent(in)    :: addclouds
  Logical,                   Intent(in)    :: addinterp  ! switch for User-Coef-User interpolations
  Logical,                   Intent(in)    :: switchrad  ! true if input is BT

  Type(profile_Type),        Intent(in)    :: profiles(nprofiles)
  Type(profile_Type),        Intent(inout) :: profiles_k(nchannels)
                                                           ! for calls from RTTOV_CLD_K and RTTOV_SCATT_K.

  Type(rttov_coef),          Intent(in)    :: coef
  Type(rttov_coef_scatt_ir), Intent(in)    :: coef_scatt_ir
  Type(rttov_optpar_ir),     Intent(in)    :: optp

  Logical,                   Intent(in)    :: calcemis(nchannels)

  Integer(Kind=jpim),        Intent(out)   :: errorstatus(nprofiles)

  Real(Kind=jprb),           Intent(inout) :: emissivity(nchannels)
  Real(Kind=jprb),           Intent(inout) :: emissivity_k(nchannels)

  Type(transmission_Type),   Intent(inout) :: transmission! in because of meme allocation
  Type(transmission_Type),   Intent(inout) :: transmission_k ! in because of meme allocation

  Type(radiance_Type),       Intent(inout) :: radiancedata! in because of meme allocation
  Type(radiance_Type),       Intent(inout), optional :: radiance_k
  Type(rttov_temp),          Intent(inout), optional :: temp, temp_k

!INTF

#include "rttov_errorreport.interface"
#include "rttov_checkinput.interface"
#include "rttov_checkinput_k.interface"
#include "rttov_profaux.interface"
#include "rttov_setgeometry.interface"
#include "rttov_setgeometry_k.interface"
#include "rttov_setpredictors_7.interface"
#include "rttov_setpredictors_8.interface"
#include "rttov_setpredictors_9.interface"
#include "rttov_opdep.interface"
#include "rttov_opdep_9.interface"
#include "rttov_opdep_9_solar.interface"
#include "rttov_transmit.interface"
#include "rttov_transmit_9_solar.interface"
#include "rttov_calcemis_ir.interface"
#include "rttov_calcemis_mw.interface"
#include "rttov_integrate.interface"
#include "rttov_profaux_k.interface"
#include "rttov_setpredictors_7_k.interface"
#include "rttov_setpredictors_8_k.interface"
#include "rttov_setpredictors_9_k.interface"
#include "rttov_opdep_k.interface"
#include "rttov_opdep_9_k.interface"
#include "rttov_opdep_9_solar_k.interface"
#include "rttov_transmit_k.interface"
#include "rttov_transmit_9_solar_k.interface"
#include "rttov_calcemis_mw_k.interface"
#include "rttov_integrate_k.interface"
#include "rttov_intavg_prof.interface"
#include "rttov_intavg_chan.interface"
#include "rttov_intavg_prof_k.interface"
#include "rttov_intavg_chan_k.interface"
#include "rttov_alloc_rad.interface"
#include "rttov_alloc_trans_scatt_ir.interface"
#include "rttov_alloc_prof.interface"
#include "rttov_alloc_transmission.interface"
#include "rttov_refsun.interface"
#include "rttov_refsun_k.interface"
#include "rttov_init_profiles.interface"
#include "rttov_init_predictor.interface"
#include "rttov_init_raytracing.interface"
#include "rttov_check_temp.interface"
#include "rttov_checkargs.interface"
#include "rttov_setpredictors_9_solar.interface"
#include "rttov_setpredictors_9_solar_k.interface" 
#include "rttov_cldstr.interface" 
#include "rttov_cldstr_k.interface" 
#include "rttov_opdpscattir.interface" 
#include "rttov_opdpscattir_k.interface" 
#include "rttov_fresnel.interface" 
#include "rttov_fresnel_k.interface"
  !local variables:
  Integer(Kind=jpim) :: i, j,prof, ii , jn, chan       ! loop index
  Integer(Kind=jpim) :: nstreams
  Integer(Kind=jpim) :: alloc_status(40)   ! memory allocation status

  Integer(Kind=jpim) :: callnum           ! which interpolator call: 1=USER->COEF, 2=COEF->USER 
  Integer(Kind=jpim) :: nvar, nc1, nc2,igas,ndim    ! no of profile elements to interpolator (USER->COEF)
  Logical            :: intvar(ngases_max)
  Logical            :: sun(nchannels)                   ! solar switch located in channel sequence

  Logical :: addcosmic          ! switch for adding temp of cosmic background
  Logical :: local_rad_k  ! false if an input K radiance is provided

  Real(Kind=jprb)    :: reflectivity(nchannels)      ! surface reflectivity
  Real(Kind=jprb)    :: reflectivity_k(nchannels)    ! K surface reflectivity

  Real(Kind=jprb)    :: fresnrefl(nchannels)
  Real(Kind=jprb)    :: fresnrefl_k(nchannels)

  Real(Kind=jprb)    :: tau_ref(profiles(1)%nlevels,nchannels)
  Real(Kind=jprb)    :: tau_ref_surf(nchannels)
  Real(Kind=jprb)    :: tau_surf(nchannels)

  Real(Kind=jprb)    :: tausun_ref(profiles(1)%nlevels,nchannels)
  Real(Kind=jprb)    :: tausun_ref_surf(nchannels)
  Real(Kind=jprb)    :: tausun_layer(profiles(1)%nlevels,nchannels)
  Real(Kind=jprb)    :: tausun_surf(nchannels)

  Real(Kind=jprb)    :: opdp_ref(profiles(1)%nlevels,nchannels) ! layer optical depth before threshold
  Real(Kind=jprb)    :: opdp_ref_COEF(coef%nlevels,nchannels) ! layer optical depth before threshold
  Real(Kind=jprb)    :: od_layer(profiles(1)%nlevels,nchannels) ! layer optical depth
  Real(Kind=jprb)    :: od_layer_COEF(coef%nlevels,nchannels) ! layer optical depth
  Real(Kind=jprb)    :: tau_layer(profiles(1)%nlevels,nchannels) ! layer transmittance

  Real(Kind=jprb)    :: opdpsun_ref(profiles(1)%nlevels,nchannels)              ! layer optical depth
  Real(Kind=jprb)    :: opdpsun_ref_COEF(coef%nlevels,nchannels)              ! layer optical depth
  Real(Kind=jprb)    :: odsun_layer(profiles(1)%nlevels,nchannels)        ! sat to layer optical depth
  Real(Kind=jprb)    :: odsun_singlelayer(profiles(1)%nlevels,nchannels)        ! sat to layer optical depth
  Real(Kind=jprb)    :: od_frac(nchannels)

! these not neede ???? anyway, refer to '_ad' in RT90 !!!!
  Real(Kind=jprb),  target :: zdeb   (5,profiles(1)%nlevels,nprofiles)
  Real(Kind=jprb),  target :: zdeb_COEF   (5,coef%nlevels,nprofiles)

!  Real(Kind=jprb),  target :: zdeb_ad(5,coef%nlevels,nprofiles)
!  Real(Kind=jprb),  target :: zdg (coef%nlevels)
!  Real(Kind=jprb),  target :: zdg_ad (coef%nlevels)
!  Real(Kind=jprb),  target :: zfac1_dg (coef%nlevels)
!  Real(Kind=jprb),  target :: zfac2_dg (coef%nlevels)
!  Real(Kind=jprb),  target :: zfac3_dg (coef%nlevels)

  Character (len=80) :: errMessage
  Character (len=8)  :: NameOfRoutine = 'rttov_k '

  Type(profile_Type)    :: profiles_COEF_ref (nprofiles)   
  Type(blob_type)       :: profiles_COEF_blob_ref

  Type(geometry_Type)   :: angles            (nprofiles)   ! geometry angles
  Type(geometry_Type)   :: angles_COEF       (nprofiles)
  
  Type(profile_aux)     :: aux_prof          (nprofiles)   ! auxillary profiles informations
  Type(profile_aux)     :: aux_prof_k        (nchannels)   ! K of above aux_prof

  Type(profile_aux)     :: aux_prof_COEF     (nprofiles)   ! auxillary profiles informations
  Type(profile_aux)     :: aux_prof_COEF_k   (nchannels)   ! K of above aux_prof

  Type(radiance_aux)    :: auxrad
  Type(radiance_aux)    :: auxrad_stream
  Type(radiance_aux)    :: auxrad_stream_k



  Type(sunglint_type)   :: sunglint(nprofiles)
  Type(sunglint_type)   :: sunglint_k (nchannels)

  Type(radiance_Type)   :: radiancedata_k          ! Local structure for K radiances


  Type(transmission_scatt_ir_type)      :: transmission_scatt_ir_stream
  Type(transmission_scatt_ir_type)      :: transmission_scatt_ir_stream_k

  Type(transmission_scatt_ir_type)      :: transmission_scatt_ir
  Type(transmission_scatt_ir_type)      :: transmission_scatt_ir_k

  Type(profile_Type)    :: profiles_k_all(nchannels)
  Type(blob_type)       :: profiles_k_all_blob

  REAL(KIND=JPRB) :: ZHOOK_HANDLE


  Integer(Kind=jpim) :: k            ! loop index just for testing)
  Type(rttov_temp), Target  :: temp1
  Type(rttov_temp), Pointer :: temp0
  Type(rttov_temp), Target  :: temp1_k
  Type(rttov_temp), Pointer :: temp0_k
  Integer(Kind=jpim)        :: temp_status, checkargs_status
  



  !- End of header ------------------------------------------------------

  !-------------
  !0. initialize
  !-------------

  IF (LHOOK) CALL DR_HOOK('RTTOV_K',0_jpim,ZHOOK_HANDLE)
  errorstatus(:)  = errorstatus_success
  alloc_status(:) = 0

  Call rttov_checkargs(  &
      & addinterp,       &
      & addsun,          &
      & addaerosl,       &
      & addclouds,       &
      & checkargs_status &
  )
  
  If( checkargs_status .eq. errorstatus_fatal ) Then
     Write( errMessage, '( "checkargs fatal error")' )
     Call Rttov_ErrorReport( errorstatus_fatal, errMessage, NameOfRoutine )
     errorstatus(:)  = errorstatus_fatal
     IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
     Return
  EndIf


  Nullify( temp0 )
  Nullify( temp0_k )

  !-----------------------------------------------------------------------
  !1. interpolator first call - input profiles from USER levs to COEF levs
  !-----------------------------------------------------------------------
  ! do immediately - for quick escape if target values are out of bounds


  Call rttov_check_temp( &
    & temp_status,                             &
    & nprofiles,                               &
    & nchannels,                               &
    & channels,                                &
    & addinterp,                               &
    & profiles(1)%nlevels,                     &
    & coef,                                    &
    & coef_scatt_ir,                           &
    & optp,                                    &
    & addsun .or. Any( profiles(:)%addsolar ), &
    & addaerosl,                               &
    & addclouds,                               &
    & 1_jpim,                                  &
    & temp0 = temp0, temp0_k = temp0_k,        &   
    & temp1 = temp1, temp1_k = temp1_k,        &   
    & temp2 = temp,  temp2_k = temp_k )  
  If( temp_status .ne. errorstatus_success ) Then
    Write( errMessage, '( "rttov_check_temp fatal error ")' )
    Call Rttov_ErrorReport( errorstatus_fatal, errMessage, NameOfRoutine )
    Return
  EndIf
  

  Call rttov_init_profiles( nprofiles, temp0%profiles_COEF )

  Do i = 1 , nprofiles
     temp0%profiles_COEF(i) % nlevels =  coef % nlevels
     temp0%profiles_COEF(i) % p(:) = coef % ref_prfl_p(:)
  Enddo

  If (addinterp) Then
    Call rttov_intavg_prof( &
            & nprofiles,                       &
            & profiles(1)%nlevels,             & !in
            & temp0%profiles_COEF(1)%nlevels,  & !in
            & profiles,                        & !in     source variables
            & temp0%profiles_COEF,             & !inout  target variables
            & errorstatus)                    !out
      If ( Any(errorstatus == errorstatus_fatal) ) Then
        IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
        Return
      End If
  Else
    Do i = 1, nprofiles
! COEF levs same as USER levs
      temp0%profiles_COEF(i)%t(:)   =    profiles(i)%t(:)
      temp0%profiles_COEF(i)%q(:)   =    profiles(i)%q(:)
      if(profiles(i) % clw_data)   temp0%profiles_COEF(i)%clw(:) =  profiles(i)%clw(:)
      if(profiles(i) % ozone_data) temp0%profiles_COEF(i)%o3(:)  =  profiles(i)%o3(:)
      if(profiles(i) % co2_data)   temp0%profiles_COEF(i)%co2(:) =  profiles(i)%co2(:)
      if(profiles(i) % n2o_data)   temp0%profiles_COEF(i)%n2o(:) =  profiles(i)%n2o(:)   
      if(profiles(i) % co_data)    temp0%profiles_COEF(i)%co(:)  =  profiles(i)%co(:)    
      if(profiles(i) % ch4_data)   temp0%profiles_COEF(i)%ch4(:) =  profiles(i)%ch4(:)
    End Do      ! over profiles
  End If

  Do i = 1, nprofiles
! complete profiles on COEF levs for surf, skin, cloud, aerosol, solar, angle
    temp0%profiles_COEF(i) % ozone_data = profiles(i) % ozone_data
    temp0%profiles_COEF(i) % co2_data   = profiles(i) % co2_data
    temp0%profiles_COEF(i) % clw_data   = profiles(i) % clw_data

    temp0%profiles_COEF(i) % n2o_data    = profiles(i) % n2o_data
    temp0%profiles_COEF(i) % co_data     = profiles(i) % co_data
    temp0%profiles_COEF(i) % ch4_data    = profiles(i) % ch4_data
    temp0%profiles_COEF(i) % aer_data    = profiles(i) % aer_data
    temp0%profiles_COEF(i) % cld_data    = profiles(i) % cld_data

    temp0%profiles_COEF(i) % S2m        = profiles(i) % S2m
    temp0%profiles_COEF(i) % skin       = profiles(i) % skin
    temp0%profiles_COEF(i) % ctp        = profiles(i) % ctp
    temp0%profiles_COEF(i) % cfraction  = profiles(i) % cfraction

! mod RWS 24/7/07
!    temp0%profiles_COEF(i) % aerosols(:,:) = profiles(i) % aerosols(:,:)
!    temp0%profiles_COEF(i) % cloud(:,:)    = profiles(i) % cloud(:,:)
!    temp0%profiles_COEF(i) % cfrac(:,:)    = profiles(i) % cfrac(:,:)

    temp0%profiles_COEF(i) % idg    = profiles(i) % idg
    temp0%profiles_COEF(i) % ish    = profiles(i) % ish

    temp0%profiles_COEF(i) % zenangle   = profiles(i) % zenangle
    temp0%profiles_COEF(i) % azangle    = profiles(i) % azangle

    temp0%profiles_COEF(i) % sunzenangle = profiles(i) % sunzenangle
    temp0%profiles_COEF(i) % sunazangle  = profiles(i) % sunazangle
    temp0%profiles_COEF(i) % latitude    = profiles(i) % latitude
    temp0%profiles_COEF(i) % elevation   = profiles(i) % elevation
    temp0%profiles_COEF(i) % addsolar    = profiles(i) % addsolar
    temp0%profiles_COEF(i) % addrefrac   = profiles(i) % addrefrac  

    temp0%profiles_COEF(i) % Be     = profiles(i) % Be  
    temp0%profiles_COEF(i) % cosbk  = profiles(i) % cosbk  
!

  End Do      ! over profiles

  Call rttov_init_raytracing( temp0%raytracing )

  !------------------------------------------------------
  !2. check input data is within suitable physical limits
  !------------------------------------------------------
  If ( apply_reg_limits ) Then

    Call rttov_alloc_prof( alloc_status(1), nprofiles, profiles_COEF_ref, &
      coef%nlevels, coef_scatt_ir, 1_jpim, blob = profiles_COEF_blob_ref )
    If( Any(alloc_status /= 0) ) Then
      errorstatus = errorstatus_fatal
      Write( errMessage, '( "allocation of profiles_COEF_ref")' )
      Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_AD',1_jpim,ZHOOK_HANDLE)
      Return
    End If

    profiles_coef_ref = temp0%profiles_COEF

  End If

  Do i = 1, nprofiles

     Call rttov_checkinput( &
           & temp0%profiles_COEF( i ),     &!in
           & coef,                         &!in
           & errorstatus(i)    )            !out 

  End Do

  ! test check input return code
  If ( any( errorstatus(:) == errorstatus_warning)  .and. verbose_checkinput_warnings ) Then
     Do i = 1, nprofiles
        If ( errorstatus(i) == errorstatus_warning ) Then
           Write( errMessage, '( "checkinput warning error for profile",i4)' ) i
           Call Rttov_ErrorReport (errorstatus_warning, errMessage, NameOfRoutine)
        End If
     End Do
  End If

  If ( any( errorstatus(:) == errorstatus_fatal ) ) Then
     Do i = 1, nprofiles
        If ( errorstatus(i) == errorstatus_fatal ) Then
           ! Some unphysical values; Do not run RTTOV
           Write( errMessage, '( "checkinput fatal error for profile",i4)' ) i
           Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
        End If
     End Do
     ! nothing processed so all profiles get the fatal error code
     ! user will know which profile
     errorstatus(:) = errorstatus_fatal
     IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
     Return
  End If

  !------------------------------------------------------------------------
  !3. determine cloud top, surface levels, ice cloud parameters
  !------------------------------------------------------------------------

  !3.1 USER levs
  !-------------
  Do i = 1, nprofiles
     If( coef % id_sensor == sensor_id_mw .or.coef % id_sensor == sensor_id_po ) Then
        Allocate( aux_prof(i) % debye_prof( 5 , profiles(1)%nlevels ), stat= alloc_status(1)  )
        If( alloc_status(1) /= 0 ) Then
           errorstatus(:) = errorstatus_fatal
           Write( errMessage, '( "allocation of aux_prof(i) % debye_prof")' )
           Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
           Return
        End If
     Endif

     if(addclouds)then
       Allocate( aux_prof(i) % dg(profiles(1)%nlevels ), stat= alloc_status(1)  )
       Allocate( aux_prof(i) % fac1_dg(profiles(1)%nlevels ), stat= alloc_status(2)  )
       Allocate( aux_prof(i) % fac2_dg(profiles(1)%nlevels ), stat= alloc_status(3)  )
       Allocate( aux_prof(i) % fac3_dg(profiles(1)%nlevels ), stat= alloc_status(4)  )
        If( alloc_status(1) /= 0 ) Then
           errorstatus(:) = errorstatus_fatal
           Write( errMessage, '( "allocation of aux_prof(i) % dg")' )
           Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
           IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
           Return
        End If
     endif

       Call rttov_profaux( &
            & addclouds,     &! in
            & profiles(i),   &! in
            & coef,          &! in
            & aux_prof(i))    ! inout
  End Do

  !3.2 COEF levs
  !-------------
  
  Do i = 1, nprofiles
     If( coef % id_sensor == sensor_id_mw  .or.coef % id_sensor == sensor_id_po ) Then
        Allocate( aux_prof_COEF(i) % debye_prof( 5 , coef%nlevels ), stat= alloc_status(1)  )
        If( alloc_status(1) /= 0 ) Then
           errorstatus(:) = errorstatus_fatal
           Write( errMessage, '( "allocation of aux_prof_COEF(i) % debye_prof")' )
           Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
           IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
           Return
        End If
     Endif

     If(addinterp) then
     Call rttov_profaux(               &
           & .false.,                  &
           & temp0%profiles_COEF(i),   &! in
           & coef,                     &! in
           & aux_prof_COEF(i))          ! inout 
     Else
       aux_prof_COEF(i) % nearestlev_surf = aux_prof(i) % nearestlev_surf
       aux_prof_COEF(i) % pfraction_surf  = aux_prof(i) % pfraction_surf
       aux_prof_COEF(i) % nearestlev_ctp  = aux_prof(i) % nearestlev_ctp 
       aux_prof_COEF(i) % pfraction_ctp   = aux_prof(i) % pfraction_ctp
       aux_prof_COEF(i) % cfraction       = aux_prof(i) % cfraction 
       If( coef % id_sensor == sensor_id_mw  .or.coef % id_sensor == sensor_id_po ) Then
         aux_prof_COEF(i) % debye_prof(:,:) = aux_prof(i) % debye_prof(:,:)
       Endif
     Endif
  End Do


  !------------------------------------------------------------------------
  !4. set up common geometric variables for the RT integration
  !------------------------------------------------------------------------

  !4.1 USER levs
  !-------------


  Call rttov_setgeometry(&
            & nprofiles,        &! in
            & profiles,         &! in
            & aux_prof,         &! in
            & coef,             &! in
            & angles,           &! out
            & temp0%raytracing ) ! out  

  !4.2 COEF levs
  !-------------

    
  If(addinterp) then
    Call rttov_setgeometry(&
            & nprofiles,              &! in
            & temp0%profiles_COEF,    &! in
            & aux_prof_COEF,          &! in
            & coef,                   &! in
            & angles_COEF,            &! out
            & temp0%raytracing_COEF )  ! out  
   Else

    angles_COEF      = angles
    temp0%raytracing_COEF % HGPL     = temp0%raytracing % HGPL
    temp0%raytracing_COEF % DAIR     = temp0%raytracing % DAIR
    temp0%raytracing_COEF % DMAIR    = temp0%raytracing % DMAIR
    temp0%raytracing_COEF % R        = temp0%raytracing % R 
    temp0%raytracing_COEF % RATOESUN = temp0%raytracing % RATOESUN
    temp0%raytracing_COEF % RATOESAT = temp0%raytracing % RATOESAT
    temp0%raytracing_COEF % ZASUN    = temp0%raytracing % ZASUN
    temp0%raytracing_COEF % ZASAT    = temp0%raytracing % ZASAT
    temp0%raytracing_COEF % INT      = temp0%raytracing % INT
    temp0%raytracing_COEF % HL       = temp0%raytracing % HL
    temp0%raytracing_COEF % PPW      = temp0%raytracing % PPW
    temp0%raytracing_COEF % DISPCO2  = temp0%raytracing % DISPCO2
    temp0%raytracing_COEF % PATHSAT  = temp0%raytracing % PATHSAT
    temp0%raytracing_COEF % PATHSUN  = temp0%raytracing % PATHSUN
    temp0%raytracing_COEF % PATHEFF  = temp0%raytracing % PATHEFF
    temp0%raytracing_COEF % LTICK    = temp0%raytracing % LTICK
  Endif

  !---------------------------------------------------
  !5. calculate atm/sol predictors - atm/sol COEF levs
  !---------------------------------------------------


  temp0%predictors%clw = 0._jprb



  If (coef%fmv_model_ver == 7) Then
     Call rttov_setpredictors_7(       &
            & nprofiles,               &
            & temp0%profiles_COEF,     &! in
            & angles_COEF,             &! in
            & coef,                    &! in
            & aux_prof_COEF,           &! in
            & temp0%predictors,        &
            & temp0%raytracing_COEF )   ! inout  (in because of mem allocation)  

  Else If (coef%fmv_model_ver == 8) Then
     Call rttov_setpredictors_8(     &
            & nprofiles,             &
            & temp0%profiles_COEF,   &! in
            & angles_COEF,           &! in
            & coef,                  &! in
            & aux_prof_COEF,         &! in
            & temp0%predictors,      &
            & temp0%raytracing_COEF ) ! inout  (in because of mem allocation)  
  Else If (coef%fmv_model_ver == 9) Then
     Call rttov_setpredictors_9_solar( &
            & nprofiles,               &
            & temp0%profiles_COEF,     &! in
            & angles_COEF,             &! in
            & coef,                    &! in
            & temp0%predictors,        &
            & temp0%raytracing_COEF ) 

     Call rttov_setpredictors_9(    &
            & nprofiles,            &
            & temp0%profiles_COEF,  &! in
            & angles_COEF,          &! in
            & coef,                 &! in
            & temp0%predictors,     &
            & temp0%raytracing_COEF )! inout  (in because of mem allocation) 

  Else
     errorstatus(:) = errorstatus_fatal
     Write( errMessage,&
            & '( "Unexpected RTTOV compatibility version number" )' ) 
     Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
     IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
     Return
  End If


  !--------------------------------------------------------------------
  !6. predict atmospheric and solar optical depths - atm/sol COEF levs 
  !--------------------------------------------------------------------

 
  If (coef%fmv_model_ver == 9) Then
    Call rttov_opdep_9(             &
       & nchannels,                 &! in
       & nprofiles,                 &! in
       & coef%nlevels,              &! in
       & channels,                  &! in
       & lprofiles,                 &! in
       & temp0%predictors,          &! in
       & aux_prof_COEF,             &! in
       & coef,                      &! in
       & temp0%opdp_path_COEF,      &! inout
       & opdp_ref_COEF)              ! inout
    Call rttov_opdep_9_solar  (     &
       & nchannels,                 &! in
       & nprofiles,                 &! in
       & coef%nlevels,              &! in
       & channels,                  &! in
       & lprofiles,                 &! in
       & profiles,                  &! in ! for solar switch only, but all profiles
       & temp0%predictors,          &! in
       & aux_prof_COEF,             &! in
       & coef,                      &! in
       & temp0%opdp_path_COEF,      &! inout
       & opdpsun_ref_COEF)           ! inout
   else
     Call rttov_opdep(        &
       & nchannels,           &! in
       & nprofiles,           &! in
       & coef%nlevels,        &! in
       & channels,            &! in
       & lprofiles,           &! in
       & temp0%predictors,    &! in
       & aux_prof_COEF,       &! in
       & coef,                &! in
       & temp0%opdp_path_COEF,&! inout
       & opdp_ref_COEF)        ! inout
   endif
   
  !--------------------------------------------------------------------------
  !7. interpolator second  call - optical depths from COEF levs to USER levs
  !--------------------------------------------------------------------------

  If (addinterp) Then
! locate any profile flags in the nchannels sequence 
! at present, only solar switch
    do j=1,nchannels
      prof=lprofiles(j)
      chan = channels(j)
      If(profiles(prof)%addsolar)Then
        If(coef%ss_val_chn(chan) .ne. 0_jpim)Then
          sun(j)=.true.     
        Else
          sun(j)=.false.   
        Endif
      Else
        sun(j)=.false.     
      Endif
    enddo
        Call rttov_intavg_chan( &
           & nchannels,                       & !in
           & sun,                             & !in
           & temp0%profiles_COEF(1)%nlevels,  & !in
           & profiles(1)%nlevels,             & !in
           & nprofiles,                       &
           & lprofiles,                       &
           & temp0%profiles_COEF,             & !inout
           & profiles,                        & !inout  only p needed
           & temp0%opdp_path_COEF,            & !inout  source variable
           & temp0%opdp_path,                 & !inout  target variable
           & errorstatus)                       !out
      If ( Any(errorstatus == errorstatus_fatal) ) Then
        IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
        Return
      End If
  Else
    temp0%opdp_path % atm_layer = temp0%opdp_path_COEF % atm_layer
    temp0%opdp_path % sun_layer = temp0%opdp_path_COEF % sun_layer
  EndIf

  !--------------------------------------------------------------------
  !8. If clouds are present,calculate the number of streams and
  !   the cloud distribution  in each stream - remains on USER levs
  !--------------------------------------------------------------------

  nstreams=0_jpim


  temp0%ircld%NSTREAM = 0_jpim
  temp0%ircld%XSTRCLR = 1._jprb

  if(addclouds)then


    Call rttov_cldstr(&
      & nprofiles,    &! in
      & profiles,     &! in
      & coef,         &! in
      & temp0%ircld,  &! out
      & addclouds,    &! in
      & nstreams)      ! out 

  endif

  !----------------------------------------------------------------------------
  !9. Calculate optical depths of aerosols and/or clouds - USER levs
  !----------------------------------------------------------------------------

  if(addaerosl)then
    do i=1,nprofiles

      Allocate( aux_prof(i)%iaernum     (profiles(1)%nlevels),stat= alloc_status(7))
      Allocate( aux_prof(i)%iaertyp  (11,profiles(1)%nlevels),stat= alloc_status(8))
      Allocate( aux_prof(i)%relhum      (profiles(1)%nlevels),stat= alloc_status(9))
      Allocate( aux_prof(i)%relhumref   (profiles(1)%nlevels),stat= alloc_status(10))

      If( Any(alloc_status /= 0) ) Then
        errorstatus(:) = errorstatus_fatal
        Write( errMessage, '( "allocation of aux_prof")' )
        Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
        IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
        Return
      End If

    enddo

  End if

  if(addaerosl.or.addclouds)then

      
    Call rttov_alloc_trans_scatt_ir( alloc_status(1), transmission_scatt_ir, &
      nchannels, profiles(1) %nlevels, 1_jpim )
    Call rttov_alloc_trans_scatt_ir( alloc_status(2), transmission_scatt_ir_stream, &
      nchannels, profiles(1) %nlevels, 1_jpim, nstreams, stream = .true. )

      If( Any(alloc_status /= 0) ) Then
        errorstatus(:) = errorstatus_fatal
        Write( errMessage, '( "allocation of trans_scatt_ir_stream")' )
        Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
        IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
        Return
      End If

      transmission_scatt_ir_stream%SSA = 0._jprb

     Call rttov_opdpscattir(            &
      & nchannels,                      &
      & nprofiles,                      &
      & coef%nlevels,                   &
      & channels,                       &
      & lprofiles,                      &
      & aux_prof,                       &
      & profiles,                       &
      & coef,                           &
      & coef_scatt_ir,                  &
      & temp0%raytracing,               &
      & transmission_scatt_ir,          &
      & transmission_scatt_ir_stream,   &
      & optp,                           &
      & temp0%ircld,                    &
      & addaerosl,                      &
      & addclouds) 
   Endif

  !----------------------------------------
  !10. calculate transmittances - USER levs
  !----------------------------------------

  Call rttov_alloc_transmission( alloc_status(1), transmission, profiles(1) % nlevels, nchannels, &
  & 1_jpim, nstreams = nstreams, calc = .true. )

  If( Any(alloc_status /= 0) ) Then
    errorstatus(:) = errorstatus_fatal
    Write( errMessage, '( "allocation of transmission")' )
    Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
    IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
    Return
  End If
      


    Call rttov_transmit(                 &
         & addaerosl,                    &! in
         & addclouds,                    &! in
         & nchannels,                    &! in
         & nprofiles,                    &! in
         & profiles(1)%nlevels,          &! in
         & channels,                     &! in
         & lprofiles,                    &! in
         & aux_prof,                     &! in
         & coef,                         &! in
         & temp0%ircld,                  &! in
         & temp0%opdp_path,              &! in
         & od_layer,                     &! out
         & transmission,                 &! out
         & transmission_scatt_ir_stream, & ! out
         & tau_ref,                      &! out  
         & tau_ref_surf,                 &! out  
         & tau_surf,                     &! out  
         & tau_layer)

  If (coef%fmv_model_ver == 9) Then
     Call rttov_transmit_9_solar(        &
         & addaerosl,                    &! in
         & addclouds,                    &! in
         & nchannels,                    &! in
         & nprofiles,                    &! in
         & profiles(1)%nlevels,          &! in
         & channels,                     &! in
         & lprofiles,                    &! in
         & profiles,                     &! in  ! for solar switch only, but all profiles
         & aux_prof,                     &! in
         & coef,                         &! in
         & temp0%raytracing,             &! in
         & temp0%ircld,                  &! in
         & temp0%opdp_path,              &! in
         & odsun_layer,                  &! out
         & odsun_singlelayer,            & ! out
         & od_frac,                      &! out
         & transmission,                 &! out
         & transmission_scatt_ir_stream, &! out
         & tausun_ref,                   & ! out
         & tausun_ref_surf,              & ! out
         & tausun_surf,                  & ! out
         & tausun_layer)                   ! out
   endif

  !-------------------------------------------------
  !11. calculate channel emissivity values - SURFACE
  !-------------------------------------------------

  If ( Any(calcemis) ) Then
     ! calculate surface emissivity for selected channels
     ! and reflectivity
     If ( coef % id_sensor == sensor_id_ir ) Then
        !Infrared
        Call rttov_calcemis_ir( &
              & profiles,      &! in
              & angles,        &! in
              & coef,          &! in
              & nchannels,     &! in
              & nprofiles,     &! in
              & channels,      &! in
              & lprofiles,     &! in
              & calcemis,      &! in
              & emissivity  )   ! inout 
        reflectivity(:) = 1 - emissivity(:)

        Elseif ( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po) Then
           !Microwave
        Call rttov_calcemis_mw ( &
              & profiles,      &! in
              & angles,        &! in
              & coef,          &! in
              & nchannels,     &! in
              & nprofiles,     &! in
              & channels,      &! in
              & lprofiles,     &! in
              & transmission,  &! in
              & calcemis,      &! in
              & emissivity,    &! inout
              & reflectivity,  &! out
              & errorstatus   ) ! out 
        If ( Any( errorstatus == errorstatus_fatal ) ) Then
           errorstatus(:) = errorstatus_fatal
           IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
           Return
        End If
     Else
        ! Hires
        Call rttov_calcemis_ir( &
              & profiles,      &! in
              & angles,        &! in
              & coef,          &! in
              & nchannels,     &! in
              & nprofiles,     &! in
              & channels,      &! in
              & lprofiles,     &! in
              & calcemis,      &! in
              & emissivity  )   ! inout 
        reflectivity(:) = 1 - emissivity(:)
     End If

     ! reflectivity for other channels
     Where( .Not. calcemis(:) )
        reflectivity(:) = 1 - emissivity(:)
     End Where

  Else
     ! reflectivity for all channels
     reflectivity(:) = 1._JPRB - emissivity(:)
  End If

  !-------------------------------------------------------
  !12. Compute Fresnel reflectance and sun glint - SURFACE
  !-------------------------------------------------------

     If ( coef % id_sensor == sensor_id_hi ) Then
       If (coef%fmv_model_ver == 9) Then

         do i=1,nprofiles
           Allocate ( sunglint  (i) % beta     (1:coef%ws_nomega),stat= alloc_status(1))
           Allocate ( sunglint  (i) % psi      (1:coef%ws_nomega),stat= alloc_status(2))
           If( Any(alloc_status /= 0) ) Then
           errorstatus(:) = errorstatus_fatal
           Write( errMessage, '( "allocation of raytracing")' )
           Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
           IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
             Return
           End If
         enddo

         call rttov_refsun( &
         & nprofiles,       &
         & profiles,        &
         & coef,            &
         & aux_prof,        &
         & sunglint,        &
         & temp0%raytracing) 

        call rttov_fresnel(&
        & nchannels,         &     
        & channels,          &
        & nprofiles,         &
        & lprofiles,         &
        & profiles,          &
        & coef,              &
        & sunglint,          &
        & fresnrefl) 
      endif
    endif

  !---------------------------------------------------------
  !13. integrate the radiative transfer equation - USER levs
  !---------------------------------------------------------

  allocate(auxrad % layer   (profiles(1) % nlevels, nchannels),stat= alloc_status(1))
  allocate(auxrad % surfair (nchannels),stat= alloc_status(2))
  allocate(auxrad % skin    (nchannels),stat= alloc_status(3))
  allocate(auxrad % cosmic  (nchannels),stat= alloc_status(4))

  allocate(auxrad_stream % up      (0:nstreams,profiles(1) % nlevels, nchannels),stat= alloc_status(5))
  allocate(auxrad_stream % down    (0:nstreams,profiles(1) % nlevels, nchannels),stat= alloc_status(6))
  Allocate(auxrad_stream % down_ref(0:nstreams,profiles(1) % nlevels,nchannels) ,stat= alloc_status(7))
  Allocate(auxrad_stream % cloudy  (0:nstreams,nchannels) ,               stat= alloc_status(17))

! If ( addcloud ) then
!    allocate(auxrad % down_cloud (profiles(1) % nlevels, nchannels),stat= alloc_status(16))
! End If

  if(addsun)then
    if(addaerosl.or.addclouds)then
      Allocate(auxrad_stream % FAC1_1    (0:nstreams,nchannels)                ,stat= alloc_status(13))
      Allocate(auxrad_stream % FAC2_1    (0:nstreams,nchannels)                ,stat= alloc_status(14))
      Allocate(auxrad_stream % FAC3_1    (0:nstreams,nchannels)                ,stat= alloc_status(15))
      Allocate(auxrad_stream % FAC4_1    (0:nstreams,nchannels)                ,stat= alloc_status(16))
      Allocate(auxrad_stream % FAC5_1    (0:nstreams,nchannels)                ,stat= alloc_status(17))
      Allocate(auxrad_stream % FAC6_1    (0:nstreams,nchannels)                ,stat= alloc_status(18))
      Allocate(auxrad_stream % FAC7_1    (0:nstreams,nchannels)                ,stat= alloc_status(19))
      Allocate(auxrad_stream % FAC1_2    (profiles(1) % nlevels,0:nstreams,nchannels) ,stat= alloc_status(20))
      Allocate(auxrad_stream % FAC2_2    (profiles(1) % nlevels,0:nstreams,nchannels) ,stat= alloc_status(21))
      Allocate(auxrad_stream % FAC3_2    (profiles(1) % nlevels,0:nstreams,nchannels) ,stat= alloc_status(22))
      Allocate(auxrad_stream % FAC4_2    (profiles(1) % nlevels,0:nstreams,nchannels) ,stat= alloc_status(23))
      Allocate(auxrad_stream % FAC5_2    (profiles(1) % nlevels,0:nstreams,nchannels) ,stat= alloc_status(24))
      Allocate(auxrad_stream % FAC6_2    (profiles(1) % nlevels,0:nstreams,nchannels) ,stat= alloc_status(25))
      Allocate(auxrad_stream % FAC7_2    (profiles(1) % nlevels,0:nstreams,nchannels) ,stat= alloc_status(26))
      Allocate(auxrad_stream % FAC1_3    (0:nstreams,nchannels)                ,stat= alloc_status(27))
      Allocate(auxrad_stream % FAC2_3    (0:nstreams,nchannels)                ,stat= alloc_status(28))
      Allocate(auxrad_stream % FAC3_3    (0:nstreams,nchannels)                ,stat= alloc_status(29))
      Allocate(auxrad_stream % FAC4_3    (0:nstreams,nchannels)                ,stat= alloc_status(30))
      Allocate(auxrad_stream % FAC5_3    (0:nstreams,nchannels)                ,stat= alloc_status(31))
      Allocate(auxrad_stream % FAC6_3    (0:nstreams,nchannels)                ,stat= alloc_status(32))
      Allocate(auxrad_stream % FAC7_3    (0:nstreams,nchannels)                ,stat= alloc_status(33))
    endif
  endif

  If( Any(alloc_status /= 0) ) Then
     errorstatus(:) = errorstatus_fatal
     Write( errMessage, '( "allocation of aux radiances")' )
     Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
     IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
     Return
  End If

  addcosmic = ( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po)
  Call rttov_integrate(                  &
        & addcosmic,                     &! in
        & addaerosl,                     &! in
        & addclouds,                     &! in
        & nchannels,                     &! in
        & nprofiles,                     &! in
        & nstreams,                      &! in
        & angles,                        &! in
        & channels,                      &! in
        & lprofiles,                     &! in
        & emissivity,                    &! in
        & reflectivity,                  &! in
        & fresnrefl,                     &! in
        & sunglint,                      &! in
        & transmission,                  &! in
        & transmission_scatt_ir_stream,  &
        & profiles,                      &! in
        & aux_prof,                      &! in
        & coef,                          &! in
        & temp0%raytracing,              &! in
        & temp0%ircld,                   &! in
        & radiancedata,                  &! inout
        & auxrad,                        &
        & auxrad_stream  ) ! inout 

  ! K matrix
  !----------------

  !---------------------------------------------
  !0. allocate and initialize local AD variables
  !---------------------------------------------

  !0.1 auxillary profile K-variables
  !----------------------------------

  ! on USER levels
  jn=profiles(1)%nlevels

  Do i = 1, nchannels
     aux_prof_k(i) % nearestlev_surf = 0   ! no meaning
     aux_prof_k(i) % pfraction_surf  = 0._JPRB  ! calculated
     aux_prof_k(i) % nearestlev_ctp  = 0   ! no meaning
     aux_prof_k(i) % pfraction_ctp   = 0._JPRB  ! calculated
     aux_prof_k(i) % cfraction       = 0._JPRB  ! calculated
     If( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po) Then
        Allocate( aux_prof_k(i) % debye_prof( 5 , jn ) ,stat= alloc_status(1))
        If( alloc_status(1) /= 0 ) Then
           errorstatus(:) = errorstatus_fatal
           Write( errMessage, '( "allocation of K debye_prof")' )
           Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
           IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
           Return
        End If
        aux_prof_k(i) % debye_prof(:,:)  = 0._JPRB
     Endif
     if(addclouds)then
       Allocate( aux_prof_k(i) % dg(jn ), stat= alloc_status(1)  )
       Allocate( aux_prof_k(i) % fac1_dg(jn ), stat= alloc_status(2)  )
       Allocate( aux_prof_k(i) % fac2_dg(jn ), stat= alloc_status(3)  )
       Allocate( aux_prof_k(i) % fac3_dg(jn ), stat= alloc_status(4)  )
       If( alloc_status(1) /= 0 ) Then
          errorstatus(:) = errorstatus_fatal
          Write( errMessage, '( "allocation of dg")' )
          Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
          IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
          Return
       End If
       aux_prof_k(i) % dg     =0._JPRB
       aux_prof_k(i) % fac1_dg=0._JPRB
       aux_prof_k(i) % fac2_dg=0._JPRB
       aux_prof_k(i) % fac3_dg=0._JPRB
     endif
  End Do     ! channel loop

  ! on COEF levels
  jn=coef%nlevels

  Do i = 1, nchannels
     aux_prof_COEF_k(i) % nearestlev_surf = 0   ! no meaning
     aux_prof_COEF_k(i) % pfraction_surf  = 0._JPRB  ! calculated
     aux_prof_COEF_k(i) % nearestlev_ctp  = 0   ! no meaning
     aux_prof_COEF_k(i) % pfraction_ctp   = 0._JPRB  ! calculated
     aux_prof_COEF_k(i) % cfraction       = 0._JPRB  ! calculated
     If( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po) Then
        Allocate( aux_prof_COEF_k(i) % debye_prof( 5 , jn ) ,stat= alloc_status(1))
        If( alloc_status(1) /= 0 ) Then
           errorstatus(:) = errorstatus_fatal
           Write( errMessage, '( "allocation of K debye_prof")' )
           Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
           IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
           Return
        End If
        aux_prof_COEF_k(i) % debye_prof(:,:)  = 0._JPRB
     Endif
  End Do     ! channel loop


  !0.2 profile_all K-variables on USER levels
  !------------------------------------------
  
  !0.2.1 USER LEVELS (all channels)
  !--------------------------------


  Call rttov_alloc_prof(alloc_status(1),nchannels,profiles_k_all, &
    & profiles(1)%nlevels,coef_scatt_ir, 1_jpim, &
    & addaerosl = addaerosl, addclouds = addclouds,init=.true.,blob = profiles_k_all_blob )

  Call rttov_init_profiles( nchannels, temp0_k%profiles_COEF )

  Do i = 1, nchannels
     j = lprofiles(i)
     temp0_k%profiles_COEF(i)%addsolar = profiles(j)%addsolar
  End Do
  
  Call rttov_init_predictor( temp0_k%predictors )

  Do i = 1, nchannels
     j = lprofiles(i)


     If(Present(radiance_k)) Then

       local_rad_k = .false.
       Do ii=1, profiles(1)%nlevels
          profiles_k_all(i) % clw (ii) = profiles_k(i) % clw (ii) 
          profiles_k_all(i) % o3  (ii) = profiles_k(i) % o3  (ii) 
          profiles_k_all(i) % co2 (ii) = profiles_k(i) % co2 (ii)
          profiles_k_all(i) % n2o (ii) = profiles_k(i) % n2o (ii)
          profiles_k_all(i) % co  (ii) = profiles_k(i) % co  (ii)
          profiles_k_all(i) % ch4 (ii) = profiles_k(i) % ch4 (ii)
          profiles_k_all(i) % t   (ii) = profiles_k(i) % t   (ii)
          profiles_k_all(i) % q   (ii) = profiles_k(i) % q   (ii)
          profiles_k_all(i) % p   (ii) = profiles_k(i) % p   (ii)
          if(addaerosl)then
            profiles_k_all(i) % aerosols(:,ii) = profiles_k(i) % aerosols(:,ii)
          endif
          if(addclouds)then
            profiles_k_all(i) % cloud   (:,ii) = profiles_k(i) % cloud   (:,ii)
            profiles_k_all(i) % cfrac   (:,ii) = profiles_k(i) % cfrac   (:,ii)
          endif
       Enddo 
       profiles_k_all(i) % zenangle   = profiles_k(i) % zenangle
       profiles_k_all(i) % azangle    = profiles_k(i) % azangle
       profiles_k_all(i) % sunzenangle= profiles_k(i) % sunzenangle
       profiles_k_all(i) % sunazangle = profiles_k(i) % sunazangle
       profiles_k_all(i) % latitude   = profiles_k(i) % latitude
       profiles_k_all(i) % elevation  = profiles_k(i) % elevation
       profiles_k_all(i) % s2m % t  = profiles_k(i) % s2m % t
       profiles_k_all(i) % s2m % u  = profiles_k(i) % s2m % u 
       profiles_k_all(i) % s2m % v  = profiles_k(i) % s2m % v 
       profiles_k_all(i) % s2m % q  = profiles_k(i) % s2m % q
       profiles_k_all(i) % s2m % o  = profiles_k(i) % s2m % o
       profiles_k_all(i) % s2m % p  = profiles_k(i) % s2m % p
       profiles_k_all(i) % s2m % wfetc = profiles_k(i) % s2m % wfetc
       profiles_k_all(i) % skin % t  = profiles_k(i) % skin % t
       profiles_k_all(i) % skin % fastem(1)  = profiles_k(i) % skin % fastem(1)
       profiles_k_all(i) % skin % fastem(2)  = profiles_k(i) % skin % fastem(2)
       profiles_k_all(i) % skin % fastem(3)  = profiles_k(i) % skin % fastem(3)
       profiles_k_all(i) % skin % fastem(4)  = profiles_k(i) % skin % fastem(4)
       profiles_k_all(i) % skin % fastem(5)  = profiles_k(i) % skin % fastem(5)
       profiles_k_all(i) % ctp  = profiles_k(i) % ctp 
       profiles_k_all(i) % cfraction  = profiles_k(i) % cfraction
     ELSE
       Do ii=1, profiles(1)%nlevels
          profiles_k_all(i) % clw (ii) = 0.0_JPRB
          profiles_k_all(i) % o3 (ii) = 0.0_JPRB
          profiles_k_all(i) % co2 (ii)= 0.0_JPRB
          profiles_k_all(i) % n2o (ii)= 0.0_JPRB
          profiles_k_all(i) % co  (ii)= 0.0_JPRB
          profiles_k_all(i) % ch4 (ii)= 0.0_JPRB
          profiles_k_all(i) % t (ii) = 0.0_JPRB
          profiles_k_all(i) % q (ii) = 0.0_JPRB
          profiles_k_all(i) % p (ii) = 0.0_JPRB
          if(addaerosl)then
            profiles_k_all(i) % aerosols(:,ii)=0.0_JPRB
          endif
          if(addclouds)then
            profiles_k_all(i) % cloud  (:,ii)=0.0_JPRB
            profiles_k_all(i) % cfrac  (:,ii)=0.0_JPRB
          endif
       Enddo
       profiles_k_all(i) % zenangle   = -1._JPRB
       profiles_k_all(i) % azangle    = -1._JPRB
       profiles_k_all(i) % sunzenangle= -1._JPRB
       profiles_k_all(i) % sunazangle = -1._JPRB
       profiles_k_all(i) % latitude   = -1._JPRB
       profiles_k_all(i) % elevation  = -1._JPRB
       profiles_k_all(i) % s2m % t  =0.0_JPRB
       profiles_k_all(i) % s2m % u  =0.0_JPRB
       profiles_k_all(i) % s2m % v  =0.0_JPRB
       profiles_k_all(i) % s2m % q  =0.0_JPRB
       profiles_k_all(i) % s2m % o  =0.0_JPRB
       profiles_k_all(i) % s2m % p  =0.0_JPRB
       profiles_k_all(i) % s2m % wfetc=0.0_JPRB
       profiles_k_all(i) % skin % t  =0.0_JPRB
       profiles_k_all(i) % skin % fastem(1)  =0.0_JPRB
       profiles_k_all(i) % skin % fastem(2)  =0.0_JPRB
       profiles_k_all(i) % skin % fastem(3)  =0.0_JPRB
       profiles_k_all(i) % skin % fastem(4)  =0.0_JPRB
       profiles_k_all(i) % skin % fastem(5)  =0.0_JPRB
       profiles_k_all(i) % ctp  =0.0_JPRB
       profiles_k_all(i) % cfraction  =0.0_JPRB
       profiles_k_all(i) % nlevels =  profiles(1) % nlevels
     END IF

  !0.2.2 COEF LEVELS (all channels)
  !--------------------------------

!  Do i = 1, nchannels
!     j = lprofiles(i)

     temp0_k%profiles_COEF(i) % ozone_Data = .False.  ! no meaning
     temp0_k%profiles_COEF(i) % co2_Data   = .False.  ! no meaning
     temp0_k%profiles_COEF(i) % n2o_Data   = .False.  ! no meaning
     temp0_k%profiles_COEF(i) % co_Data    = .False.  ! no meaning
     temp0_k%profiles_COEF(i) % ch4_Data   = .False.  ! no meaning
     temp0_k%profiles_COEF(i) % clw_Data   = .False.  ! no meaning
     temp0_k%profiles_COEF(i) % zenangle   = -1       ! no meaning
     temp0_k%profiles_COEF(i) % azangle    = -1       ! no meaning
     temp0_k%profiles_COEF(i) % sunzenangle= -1       ! no meaning
     temp0_k%profiles_COEF(i) % sunazangle = -1       ! no meaning
     temp0_k%profiles_COEF(i) % latitude   = -1       ! no meaning
     temp0_k%profiles_COEF(i) % elevation  = -1       ! no meaning
     temp0_k%profiles_COEF(i) % p(:)   = 0._JPRB ! no AD on pressure levels
     temp0_k%profiles_COEF(i) % t(:)   = 0._JPRB ! temperarure
     temp0_k%profiles_COEF(i) % q(:)   = 0._JPRB ! WV
     temp0_k%profiles_COEF(i) % o3(:)  = 0._JPRB ! O3 ppmv
     temp0_k%profiles_COEF(i) % clw(:) = 0._JPRB ! clw
     temp0_k%profiles_COEF(i) % co2(:) = 0._JPRB ! co2
     temp0_k%profiles_COEF(i) % n2o(:) = 0._JPRB ! N2O ppmv
     temp0_k%profiles_COEF(i) % co(:)  = 0._JPRB ! CO ppmv
     temp0_k%profiles_COEF(i) % ch4(:) = 0._JPRB ! CH4 ppmv

!     if(addaerosl)then
!       temp0_k%profiles_COEF(i) % aerosols(:,:) = 0._JPRB 
!     endif

     temp0_k%profiles_COEF(i) % s2m % t = 0._JPRB! temperarure
     temp0_k%profiles_COEF(i) % s2m % q = 0._JPRB! WV
     temp0_k%profiles_COEF(i) % s2m % o = 0._JPRB! O3
     temp0_k%profiles_COEF(i) % s2m % p = 0._JPRB! pressure
     temp0_k%profiles_COEF(i) % s2m % u = 0._JPRB! wind components
     temp0_k%profiles_COEF(i) % s2m % v = 0._JPRB! wind components
     temp0_k%profiles_COEF(i) % s2m % wfetc = 0._JPRB! wind fetch 

     temp0_k%profiles_COEF(i) % skin % surftype = -1  ! no meaning
     temp0_k%profiles_COEF(i) % skin % t        = 0._JPRB  ! on temperarure
     temp0_k%profiles_COEF(i) % skin % fastem   = 0._JPRB  ! Fastem

     temp0_k%profiles_COEF(i) % ctp       = 0._JPRB  ! cloud top pressure
     temp0_k%profiles_COEF(i) % cfraction = 0._JPRB  ! cloud fraction

  !0.3 predictor K-variables on COEF levels
  !------------------------------------------


     temp0_k%predictors % nlevels  = temp0%predictors % nlevels
     temp0_k%predictors % nmixed   = temp0%predictors % nmixed
     temp0_k%predictors % nwater   = temp0%predictors % nwater
     temp0_k%predictors % nozone   = temp0%predictors % nozone
     temp0_k%predictors % nwvcont  = temp0%predictors % nwvcont
     temp0_k%predictors % nco2     = temp0%predictors % nco2
     temp0_k%predictors % nn2o     = temp0%predictors % nn2o
     temp0_k%predictors % nco      = temp0%predictors % nco 
     temp0_k%predictors % nch4     = temp0%predictors % nch4
     temp0_k%predictors % ncloud   = temp0%predictors % ncloud

  End Do       ! channel loop

  reflectivity_k(:) = 0._JPRB

  !0.4 raytracing K-variables
  !--------------------------


  Call rttov_init_raytracing( temp0_k%raytracing )
  Call rttov_init_raytracing( temp0_k%raytracing_COEF )
  

  Do i=1,nchannels

  ! other 
    sunglint_k (i)  % GLINT       = 0._JPRB
    sunglint_k (i)  % OMEGA       = 0._JPRB
    temp0_k%ircld  % XSTRCLR(i)   = 0._JPRB
    fresnrefl_k(i)                = 0._JPRB

  !0.5 ircld K-variables on USER levels
  !------------------------------------

    If(addclouds) then
      temp0_k%ircld% xstr  (:,i)  =0._JPRB
    End If

    If(addaerosl)then

      Allocate( aux_prof_k(i)%relhum   (profiles(1)%nlevels),stat= alloc_status(9))

      If( Any(alloc_status /= 0) ) Then
        errorstatus(:) = errorstatus_fatal
        Write( errMessage, '( "allocation of ircld_k")' )
        Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
        IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
        Return
      End If

      temp0_k%ircld % tave(:,i)    =0._jprb
      temp0_k%ircld % wmixave(:,i) =0._jprb
      temp0_k%ircld % esw(:,i)     =0._jprb
      temp0_k%ircld % esi(:,i)     =0._jprb
      temp0_k%ircld % ppv(:,i)     =0._jprb
      aux_prof_k(i)%relhum =0._jprb
    Endif

  End Do   ! channel loop

  !0.6 path opdep K-variables on COEF levels
  !-----------------------------------------


  temp0_k%opdp_path%atm_layer = 0._jprb
  temp0_k%opdp_path%sun_layer = 0._jprb
  temp0_k%opdp_path_COEF%atm_layer = 0._jprb
  temp0_k%opdp_path_COEF%sun_layer = 0._jprb


  !0.7 transmit K-variables on USER levels
  !---------------------------------------



  Call rttov_alloc_transmission( alloc_status(1), transmission_k, profiles(1) % nlevels, nchannels, &
  & 1_jpim, nstreams = nstreams, init = .true., calc = .true. )

  If( Any(alloc_status /= 0) ) Then
    errorstatus(:) = errorstatus_fatal
    Write( errMessage, '( "allocation of transmission_k")' )
    Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
    IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
    Return
  End If



  if(addaerosl.or.addclouds)then

    Call rttov_alloc_trans_scatt_ir( alloc_status(1), transmission_scatt_ir_k, &
      nchannels, profiles(1) %nlevels, 1_jpim, init = .true. )

    Call rttov_alloc_trans_scatt_ir( alloc_status(2), transmission_scatt_ir_stream_k, &
      nchannels, profiles(1) %nlevels, 1_jpim, nstreams, stream = .true., init = .true. )


    If( Any(alloc_status /= 0) ) Then
      errorstatus(:) = errorstatus_fatal
      Write( errMessage, '( "allocation of trans_scatt_ir_k")' )
      Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
      Return
    End If


  Endif


  !0.8 auxrad_stream K-variables on USER levels
  !--------------------------------------------

    Allocate(auxrad_stream_k % up      (0:nstreams,profiles(1) % nlevels, nchannels) ,stat= alloc_status(5))
    Allocate(auxrad_stream_k % down    (0:nstreams,profiles(1) % nlevels, nchannels) ,stat= alloc_status(6))
    Allocate(auxrad_stream_k % cloudy  (0:nstreams,nchannels) ,stat= alloc_status(16))

    Allocate(auxrad_stream   % rad_tmp           (0:nstreams,nchannels) ,stat= alloc_status(5))
    Allocate(auxrad_stream   % meanrad_up        (0:nstreams,nchannels) ,stat= alloc_status(6))
    Allocate(auxrad_stream   % meanrad_up_2d     (0:nstreams,nchannels,profiles(1) % nlevels) ,stat= alloc_status(7))
    Allocate(auxrad_stream   % meanrad_down      (0:nstreams,nchannels) ,stat= alloc_status(8))
    Allocate(auxrad_stream   % meanrad_down_ref  (0:nstreams,nchannels) ,stat= alloc_status(9))

    if(addsun)then
      if(addaerosl.or.addclouds)then
        Allocate(auxrad_stream_k % FAC1_1    (0:nstreams,nchannels) ,stat= alloc_status(17))
        Allocate(auxrad_stream_k % FAC2_1    (0:nstreams,nchannels) ,stat= alloc_status(18))
        Allocate(auxrad_stream_k % FAC3_1    (0:nstreams,nchannels) ,stat= alloc_status(19))
        Allocate(auxrad_stream_k % FAC4_1    (0:nstreams,nchannels ) ,stat= alloc_status(20))
        Allocate(auxrad_stream_k % FAC5_1    (0:nstreams,nchannels ) ,stat= alloc_status(21))
        Allocate(auxrad_stream_k % FAC6_1    (0:nstreams,nchannels ) ,stat= alloc_status(22))
        Allocate(auxrad_stream_k % FAC7_1    (0:nstreams,nchannels) ,stat= alloc_status(23))
        Allocate(auxrad_stream_k % FAC1_2    (profiles(1) % nlevels,0:nstreams,nchannels) ,stat= alloc_status(24))
        Allocate(auxrad_stream_k % FAC2_2    (profiles(1) % nlevels,0:nstreams,nchannels) ,stat= alloc_status(25))
        Allocate(auxrad_stream_k % FAC3_2    (profiles(1) % nlevels,0:nstreams,nchannels) ,stat= alloc_status(26))
        Allocate(auxrad_stream_k % FAC4_2    (profiles(1) % nlevels,0:nstreams,nchannels) ,stat= alloc_status(27))
        Allocate(auxrad_stream_k % FAC5_2    (profiles(1) % nlevels,0:nstreams,nchannels) ,stat= alloc_status(28))
        Allocate(auxrad_stream_k % FAC6_2    (profiles(1) % nlevels,0:nstreams,nchannels) ,stat= alloc_status(29))
        Allocate(auxrad_stream_k % FAC7_2    (profiles(1) % nlevels,0:nstreams,nchannels) ,stat= alloc_status(30))
        Allocate(auxrad_stream_k % FAC1_3    (0:nstreams,nchannels) ,stat= alloc_status(31))
        Allocate(auxrad_stream_k % FAC2_3    (0:nstreams,nchannels) ,stat= alloc_status(32))
        Allocate(auxrad_stream_k % FAC3_3    (0:nstreams,nchannels) ,stat= alloc_status(33))
        Allocate(auxrad_stream_k % FAC4_3    (0:nstreams,nchannels ),stat= alloc_status(34))
        Allocate(auxrad_stream_k % FAC5_3    (0:nstreams,nchannels ),stat= alloc_status(35))
        Allocate(auxrad_stream_k % FAC6_3    (0:nstreams,nchannels ),stat= alloc_status(36))
        Allocate(auxrad_stream_k % FAC7_3    (0:nstreams,nchannels) ,stat= alloc_status(37))
      endif
    endif

If( Any(alloc_status /= 0) ) Then
        errorstatus(:) = errorstatus_fatal
        Write( errMessage, '( "allocation of auxrad_stream_k")' )
        Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
        IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
        Return
    endif

    auxrad_stream_k % up     =0._JPRB
    auxrad_stream_k % down   =0._JPRB
    auxrad_stream_k % cloudy =0._JPRB

    if(addsun)then
      if(addaerosl.or.addclouds)then
        auxrad_stream_k % FAC1_1   =0._JPRB
        auxrad_stream_k % FAC2_1   =0._JPRB
        auxrad_stream_k % FAC3_1   =0._JPRB
        auxrad_stream_k % FAC4_1   =0._JPRB
        auxrad_stream_k % FAC5_1   =0._JPRB
        auxrad_stream_k % FAC6_1   =0._JPRB
        auxrad_stream_k % FAC7_1   =0._JPRB
        auxrad_stream_k % FAC1_2   =0._JPRB
        auxrad_stream_k % FAC2_2   =0._JPRB
        auxrad_stream_k % FAC3_2   =0._JPRB
        auxrad_stream_k % FAC4_2   =0._JPRB
        auxrad_stream_k % FAC5_2   =0._JPRB
        auxrad_stream_k % FAC6_2   =0._JPRB
        auxrad_stream_k % FAC7_2   =0._JPRB
        auxrad_stream_k % FAC1_3   =0._JPRB
        auxrad_stream_k % FAC2_3   =0._JPRB
        auxrad_stream_k % FAC3_3   =0._JPRB
        auxrad_stream_k % FAC4_3   =0._JPRB
        auxrad_stream_k % FAC5_3   =0._JPRB
        auxrad_stream_k % FAC6_3   =0._JPRB
        auxrad_stream_k % FAC7_3   =0._JPRB
      endif
    endif

  !--------------------------------------------------
  !1. K of radiative transfer integration - USER levs
  !--------------------------------------------------
  If(Present(radiance_k)) Then

     ! use the subroutine argument for K radiances
     addcosmic = ( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po)
     Call rttov_integrate_k(                 &
           & addcosmic,                      &! in
           & addaerosl,                      &
           & addclouds,                      &
           & switchrad,                      &! in
           & nchannels,                      &! in
           & nprofiles,                      &! in
           & nstreams,                       &! in
           & angles,                         &! in
           & channels,                       &! in
           & lprofiles,                      &! in
           & emissivity,                     &! in
           & emissivity_k,                   &! inout
           & reflectivity,                   &! in
           & reflectivity_k,                 &! inout
           & fresnrefl,                      &
           & fresnrefl_k,                    &
           & sunglint,                       &!  in
           & sunglint_k,                     &
           & transmission,                   &! in
           & transmission_k,                 &! inout
           & transmission_scatt_ir_stream,   &
           & transmission_scatt_ir_stream_k, &
           & profiles,                       &! in
           & profiles_k_all,                 &! inout  (input only due to mem alloc)
           & aux_prof,                       &! in
           & aux_prof_k,                     &! inout
           & coef,                           &! in
           & temp0%raytracing,               &! in
           & temp0_k%raytracing,             &! in
           & temp0%ircld,                    &! in
           & temp0_k%ircld,                  &! in
           & radiancedata,                   &! in
           & auxrad ,                        &! in
           & auxrad_stream,                  &! in
           & auxrad_stream_k,                &! in
           & radiance_k      )                ! inout  (output if converstion Bt -> rad) 
  Else

     local_rad_k = .true.

     ! create a local structure for input K radiances

     Call rttov_alloc_rad( alloc_status(1), nchannels, radiancedata_k, profiles(1)%nlevels, 1_jpim )

     If( Any(alloc_status /= 0) ) Then
        errorstatus(:) = errorstatus_fatal
        Write( errMessage, '( "allocation of K radiances")' )
        Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
        IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
        Return
     End If

     radiancedata_k % clear      = 0._JPRB
     radiancedata_k % cloudy     = 0._JPRB
     radiancedata_k % bt_clear   = 0._JPRB
     radiancedata_k % upclear    = 0._JPRB
     radiancedata_k % reflclear  = 0._JPRB
     radiancedata_k % overcast   = 0._JPRB
     radiancedata_k % bt         = 1._JPRB
     radiancedata_k % total      = 1._JPRB

     addcosmic = ( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po)
     Call rttov_integrate_k(                 &
           & addcosmic,                      &! in
           & addaerosl,                      &
           & addclouds,                      &
           & switchrad,                      &! in
           & nchannels,                      &! in
           & nprofiles,                      &! in
           & nstreams,                       &! in
           & angles,                         &! in
           & channels,                       &! in
           & lprofiles,                      &! in
           & emissivity,                     &! in
           & emissivity_k,                   &! inout
           & reflectivity,                   &! in
           & reflectivity_k,                 &! inout
           & fresnrefl,                      &
           & fresnrefl_k,                    &
           & sunglint,                       &!  in
           & sunglint_k,                     &
           & transmission,                   &! in
           & transmission_k,                 &! inout
           & transmission_scatt_ir_stream,   &
           & transmission_scatt_ir_stream_k, &
           & profiles,                       &! in
           & profiles_k_all,                 &! inout  (input only due to mem alloc)
           & aux_prof,                       &! in
           & aux_prof_k,                     &! inout
           & coef,                           &! in
           & temp0%raytracing,               &! i
           & temp0_k%raytracing,             &! in
           & temp0%ircld,                    &! in
           & temp0_k%ircld,                  &
           & radiancedata,                   &! in
           & auxrad ,                        &! in
           & auxrad_stream,                  &! in
           & auxrad_stream_k,                &
           & radiancedata_k  )                ! inout  (output if converstion Bt -> rad) 
  Endif

  !--------------------------------------
  !2. K of channel emissivities - SURFACE
  !--------------------------------------

     If ( coef % id_sensor == sensor_id_hi ) Then
       If (coef%fmv_model_ver == 9) Then
         call rttov_fresnel_k(&
         & nchannels,         &
         & channels,          &
         & nprofiles,         &
         & lprofiles,         &
         & profiles,          &
         & coef,              &
         & sunglint,          &
         & sunglint_k,        &
         & fresnrefl,         &
         & fresnrefl_k) 

         do i=1,nchannels
           Allocate ( sunglint_k(i) % beta     (1:coef%ws_nomega),stat= alloc_status(1))
           Allocate ( sunglint_k(i) % psi      (1:coef%ws_nomega),stat= alloc_status(2))
           If( Any(alloc_status /= 0) ) Then
             errorstatus(:) = errorstatus_fatal
             Write( errMessage, '( "allocation of raytracing")' )
             Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
             IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
             Return
           End If
         enddo

         call rttov_refsun_k(     &
         & nprofiles,             &
         & channels,              &
         & nchannels,             &
         & lprofiles,             &
         & profiles,              &
         & profiles_k_all,        &
         & coef,                  &
         & aux_prof,              &
         & sunglint,              &
         & sunglint_k,            &
         & temp0%raytracing,      &
         & temp0_k%raytracing) 

       endif
     endif

  If ( Any(calcemis) ) Then
     ! calculate surface emissivity for selected channels
     ! and reflectivity
     If ( coef % id_sensor == sensor_id_ir ) Then
        !Infrared
        emissivity_k(:) = -reflectivity_k(:) + emissivity_k(:)

        Elseif ( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po ) Then
           !Microwave
        Call rttov_calcemis_mw_k ( &
              & profiles,          &! in
              & profiles_k_all,    &! inout
              & angles,            &! in
              & coef,              &! in
              & nchannels,         &! in
              & nprofiles,         &! in
              & channels,          &! in
              & lprofiles,         &! in
              & transmission,      &! in
              & transmission_k,    &! inout
              & calcemis,          &! in
              & emissivity_k,      &! inout
              & reflectivity_k   )  ! inout 
     Else
        ! Hires
        emissivity_k(:) = -reflectivity_k(:) + emissivity_k(:)
     End If

     ! reflectivity for other channels
     Where( .Not. calcemis(:) )
        emissivity_k(:) = -reflectivity_k(:) + emissivity_k(:)
     End Where

  Else
     ! reflectivity for all channels
     emissivity_k(:) = -reflectivity_k(:) + emissivity_k(:)
  End If

  !----------------------------------
  !3. K of transmittances - USER levs
  !----------------------------------

  If (coef%fmv_model_ver == 9) Then
   Call rttov_transmit_9_solar_k(          &
         & addaerosl,                      &! in
         & addclouds,                      &! in
         & nchannels,                      &! in
         & nprofiles,                      &! in
         & profiles(1)%nlevels,            &! in
         & channels,                       &! in
         & lprofiles,                      &! in
         & profiles,                       &! in    ! for solar switch
         & aux_prof,                       &! in
         & aux_prof_k,                     &! inout
         & coef,                           &! in
         & temp0%raytracing,               &! in
         & temp0_k%raytracing,             &! in
         & temp0%ircld,                    &!
         & temp0%opdp_path,                &! in
         & temp0_k%opdp_path,              &! in
         & odsun_layer,                    & ! out
         & odsun_singlelayer,              & ! out
         & od_frac,                        &! out
         & transmission,                   &! in
         & transmission_k,                 &
         & transmission_scatt_ir_stream,   &
         & transmission_scatt_ir_stream_k, &
         & tausun_ref,                     & ! in
         & tausun_ref_surf,                & ! in
         & tausun_layer,                   & ! in
         & tausun_surf)                      ! in
  endif
    Call rttov_transmit_k(                 &
         & addaerosl,                      &! in
         & addclouds,                      &
         & nchannels,                      &! in
         & nprofiles,                      &! in
         & profiles(1)%nlevels,            &! in
         & channels,                       &! in
         & lprofiles,                      &! in
         & aux_prof,                       &! in
         & aux_prof_k,                     &! inout
         & coef,                           &! in
         & temp0%ircld,                    &
         & temp0%opdp_path,                &! inout
         & temp0_k%opdp_path,              &! inout
         & od_layer,                       &! in
         & transmission,                   &! in
         & transmission_k,                 &
         & transmission_scatt_ir_stream ,  &! in
         & transmission_scatt_ir_stream_k, & ! inout 
         & tau_ref,                        &! in         
         & tau_ref_surf,                   &! in 
         & tau_surf,                       &! in 
         & tau_layer)                       ! in

 !------------------------------------------------------------
  !4. K of optical depths of aerosols and/or clouds - USER levs
  !------------------------------------------------------------

   If (addaerosl.or.addclouds)Then
    Call rttov_opdpscattir_k(&
      & nchannels,                      &
      & nprofiles,                      &
      & profiles(1)%nlevels,            &
      & channels,                       &
      & lprofiles,                      &
      & aux_prof,                       &
      & aux_prof_k,                     &
      & profiles,                       &
      & profiles_k_all ,                &
      & coef,                           &
      & coef_scatt_ir,                  &
      & temp0%raytracing,               &
      & temp0_k%raytracing,             &
      & transmission_scatt_ir,          &
      & transmission_scatt_ir_k,        &
      & transmission_scatt_ir_stream,   &
      & transmission_scatt_ir_stream_k, &
      & optp,                           &
      & temp0%ircld,                    &
      & temp0_k%ircld,                  &
      & addaerosl,                      &
      & addclouds) 
  Endif


  !---------------------------------------------------
  !5. K of cloud streams and distributions - USER levs
  !---------------------------------------------------

  if(addclouds)then

    Call rttov_cldstr_k(&
      & nprofiles,      &! in
      & nchannels,      &
      & lprofiles,      &
      & profiles,       &! in
      & profiles_k_all, &
      & coef,           &! in
      & temp0%ircld,    &
      & temp0_k%ircld,  &
      & addclouds)         ! out 

    temp0_k%ircld%XSTRCLR = 0._jprb
  endif


  !-----------------------------------
  !6. K of optical depth interpolation 
  !-----------------------------------

    If (addinterp) Then
      Call rttov_intavg_chan_k(                  &
           & nchannels,                          & !in 
           & sun,                                & !in
           & temp0%profiles_COEF(1)%nlevels,     & !in
           & profiles(1)%nlevels,                & !in
	   & nprofiles,                          & !in 
	   & lprofiles,                          & !in 
           & temp0%profiles_COEF,                & !in  only p used this call 
           & profiles,                           & !in  only p used this call 
           & profiles_k_all,                     & !in  only p used this call 
           & temp0%opdp_path_COEF,               & !in  
           & temp0_k%opdp_path_COEF,             & !inout
           & temp0%opdp_path,                    & !in  
           & temp0_k%opdp_path,                  & !in
           & errorstatus)                          !out
      If ( Any(errorstatus == errorstatus_fatal) ) Then
        IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
        Return
      End If
  Else
     temp0_k%opdp_path_COEF%atm_layer(:,:) = temp0_k%opdp_path_COEF%atm_layer(:,:) &
       + temp0_k%opdp_path%atm_layer(:,:)
     temp0_k%opdp_path_COEF%sun_layer(:,:) = temp0_k%opdp_path_COEF%sun_layer(:,:) &
       + temp0_k%opdp_path%sun_layer(:,:)
  EndIf

  !--------------------------------------------------------
  !7. K of atmospheric and solar optical depths - COEF levs 
  !--------------------------------------------------------

  ! optical depth arrays allocated at start of K-code


  If (coef%fmv_model_ver == 9) Then
    Call rttov_opdep_9_solar_k  ( &
       & nchannels,               &! in
       & nprofiles,               &! in
       & coef%nlevels,            &! in
       & channels,                &! in
       & lprofiles,               &! in
       & profiles,                &! in    ! for solar switch 
       & temp0%predictors,        &! in
       & temp0_k%predictors,      &! inout
       & aux_prof_COEF,           &! in
       & aux_prof_COEF_k,         &! inout
       & coef,                    &! in
       & temp0%opdp_path_COEF,    &! in
       & temp0_k%opdp_path_COEF,  &! inout
       & opdpsun_ref_COEF)         ! in
    Call rttov_opdep_9_k(         &
       & nchannels,               &! in
       & nprofiles,               &! in
       & coef%nlevels,            &! in
       & channels,                &! in
       & lprofiles,               &! in
       & temp0%predictors,        &! in
       & temp0_k%predictors,      &! inout
       & aux_prof_COEF,           &! in
       & aux_prof_COEF_k,         &! inout
       & coef,                    &! in
       & temp0%opdp_path_COEF,    &! in
       & temp0_k%opdp_path_COEF,  &! inout
       & opdp_ref_COEF)            ! in
   else
     Call rttov_opdep_k(            &
       & nchannels,                 &! in
       & nprofiles,                 &! in
       & coef%nlevels,              &! in
       & channels,                  &! in
       & lprofiles,                 &! in
       & temp0%predictors,          &! in
       & temp0_k%predictors,        &! inout
       & aux_prof_COEF,             &! in
       & aux_prof_COEF_k,           &! inout
       & coef,                      &! in
       & temp0%opdp_path_COEF,      &! in
       & temp0_k%opdp_path_COEF,    &! inout
       & opdp_ref_COEF)              ! in
   endif

  !-------------------------------------------------------
  !8. K of RTTOV-7 RTTOV-8 RTTOV-9 predictors - COEF levs 
  !-------------------------------------------------------

  If (coef%fmv_model_ver == 7) Then
     Call rttov_setpredictors_7_k(     &
           & nchannels,                &! in
           & nprofiles,                &! in
           & coef%nlevels,             &! in
           & angles_COEF,              &! in
           & lprofiles,                &! in
           & temp0%profiles_COEF,      &! in
           & temp0_k%profiles_COEF,    &! inout
           & coef,                     &! in
           & aux_prof_COEF,            &! in
           & temp0%predictors,         &! in
           & temp0_k%predictors,       &! inout
           & temp0%raytracing_COEF,    &
           & temp0_k%raytracing_COEF )  ! in  
  Else If (coef%fmv_model_ver == 8) Then
     Call rttov_setpredictors_8_k(     &
           & nchannels,                &! in
           & nprofiles,                &! in
           & coef%nlevels,             &! in
           & angles_COEF,              &! in
           & lprofiles,                &! in
           & temp0%profiles_COEF,      &! in
           & temp0_k%profiles_COEF,    &! inout
           & coef,                     &! in
           & aux_prof_COEF,            &! in
           & temp0%predictors,         &! in
           & temp0_k%predictors,       &! inout
           & temp0%raytracing_COEF,    &
           & temp0_k%raytracing_COEF )  ! in  
  Else If (coef%fmv_model_ver == 9) Then

     Call rttov_setpredictors_9_solar_k(      &
               & nchannels,                   &! in
               & nprofiles,                   &! in
               & coef%nlevels,                &! in
               & lprofiles,                   &! in
               & temp0%profiles_COEF,         &! in
               & temp0_k%profiles_COEF,       &! inout
               & coef,                        &! in
               & temp0%predictors,            &! in
               & temp0_k%predictors,          &
               & temp0%raytracing_COEF,       &
               & temp0_k%raytracing_COEF )     ! in 
     Call rttov_setpredictors_9_k(           &
               & nchannels,                  &! in
               & nprofiles,                  &! in
               & coef%nlevels,               &! in
               & angles_COEF,                &! in
               & lprofiles,                  &! in
               & temp0%profiles_COEF,        &! in
               & temp0_k%profiles_COEF,      &! inout
               & coef,                       &! in
               & temp0%predictors,           &! in
               & temp0_k%predictors,         &
               & temp0%raytracing_COEF,      &
               & temp0_k%raytracing_COEF )    ! in 
  End If

  !------------------------------------------------------
  !9. K of common geometric set-up for the RT integration
  !------------------------------------------------------
 
  !9.1 COEF levs
  !-------------
 
  If(addinterp) then
      Call rttov_setgeometry_k(        &
             & nchannels,              &
             & nprofiles,              &
             & lprofiles,              &
             & temp0%profiles_COEF,    &
             & temp0_k%profiles_COEF,  &
             & aux_prof_COEF,          &! in
             & coef,                   &! in
             & angles_COEF,            &
             & temp0%raytracing_COEF,  &
             & temp0_k%raytracing_COEF   )     ! out  
   Else

     temp0_k%raytracing % HGPL     = temp0_k%raytracing % HGPL     + temp0_k%raytracing_COEF % HGPL
     temp0_k%raytracing % DAIR     = temp0_k%raytracing % DAIR     + temp0_k%raytracing_COEF % DAIR
     temp0_k%raytracing % DMAIR    = temp0_k%raytracing % DMAIR    + temp0_k%raytracing_COEF % DMAIR
     temp0_k%raytracing % R        = temp0_k%raytracing % R        + temp0_k%raytracing_COEF % R  
     temp0_k%raytracing % RATOESUN = temp0_k%raytracing % RATOESUN + temp0_k%raytracing_COEF % RATOESUN
     temp0_k%raytracing % RATOESAT = temp0_k%raytracing % RATOESAT + temp0_k%raytracing_COEF % RATOESAT
     temp0_k%raytracing % ZASUN    = temp0_k%raytracing % ZASUN    + temp0_k%raytracing_COEF % ZASUN 
     temp0_k%raytracing % ZASAT    = temp0_k%raytracing % ZASAT    + temp0_k%raytracing_COEF % ZASAT
     temp0_k%raytracing % INT      = temp0_k%raytracing % INT      + temp0_k%raytracing_COEF % INT
     temp0_k%raytracing % HL       = temp0_k%raytracing % HL       + temp0_k%raytracing_COEF % HL
     temp0_k%raytracing % PPW      = temp0_k%raytracing % PPW      + temp0_k%raytracing_COEF % PPW
     temp0_k%raytracing % DISPCO2  = temp0_k%raytracing % DISPCO2  + temp0_k%raytracing_COEF % DISPCO2
     temp0_k%raytracing % PATHSAT  = temp0_k%raytracing % PATHSAT  + temp0_k%raytracing_COEF % PATHSAT
     temp0_k%raytracing % PATHSUN  = temp0_k%raytracing % PATHSUN  + temp0_k%raytracing_COEF % PATHSUN
     temp0_k%raytracing % PATHEFF  = temp0_k%raytracing % PATHEFF  + temp0_k%raytracing_COEF % PATHEFF
     temp0_k%raytracing % LTICK    = temp0_k%raytracing % LTICK    + temp0_k%raytracing_COEF % LTICK

  Endif



  !9.1 USER levs
  !-------------

  Call rttov_setgeometry_k(        &
            & nchannels,           &
            & nprofiles,           &
            & lprofiles,           &
            & profiles,            &
            & profiles_k_all,      &
            & aux_prof,            &! in
            & coef,                &! in
            & angles,              &
            & temp0%raytracing,    &
            & temp0_k%raytracing   )! out 

  !---------------------------------------------------------
  !10. AD of cloud top, surface levels, ice cloud parameters
  !---------------------------------------------------------

  !10.1 COEF levs
  !-------------

  If(addinterp) then
  Call rttov_profaux_k(           &
      & .false.,                  &! in
      & nchannels,                &! in
      & nprofiles,                &! in
      & lprofiles,                &! in
      & temp0%profiles_COEF,      &! in
      & temp0_k%profiles_COEF,    &! inout - NB interpolated from profiles_k_all
      & coef,                     &! in
      & aux_prof_COEF,            &! in
      & aux_prof_COEF_k   )        ! inout 

     Else 
  Do i = 1, nchannels
       aux_prof_k(i) % nearestlev_surf = aux_prof_k(i) % nearestlev_surf &
                                           & + aux_prof_COEF_k(i) % nearestlev_surf
       aux_prof_k(i) % pfraction_surf  = aux_prof_k(i) % pfraction_surf &
                                           & + aux_prof_COEF_k(i) % pfraction_surf
       aux_prof_k(i) % nearestlev_ctp  = aux_prof_k(i) % nearestlev_ctp &
                                           & + aux_prof_COEF_k(i) % nearestlev_ctp 
       aux_prof_k(i) % pfraction_ctp   = aux_prof_k(i) % pfraction_ctp &
                                           & + aux_prof_COEF_k(i) % pfraction_ctp
       aux_prof_k(i) % cfraction       = aux_prof_k(i) % cfraction &
                                           & + aux_prof_COEF_k(i) % cfraction 
       If( coef % id_sensor == sensor_id_mw  .or.coef % id_sensor == sensor_id_po ) Then
         aux_prof_k(i) % debye_prof(:,:) = aux_prof_k(i) % debye_prof(:,:) &
	                                   & + aux_prof_COEF_k(i) % debye_prof(:,:)
       Endif
   End Do
 Endif

  !10.2 USER levs
  !-------------
  Call rttov_profaux_k( &
      & addclouds,     &! in
      & nchannels,     &! in
      & nprofiles,     &! in
      & lprofiles,     &! in
      & profiles,      &! in
      & profiles_k_all,&! inout
      & coef,          &! in
      & aux_prof,      &! in
      & aux_prof_k   )  ! inout 

  !------------------------------------------------------------------
  !11 K of check input data is within suitable physical limits - COEF levs
  !------------------------------------------------------------------
 
  If ( apply_reg_limits ) Then
    Do i = 1, nchannels
      j = lprofiles(i)
      Call rttov_checkinput_k( &
           & profiles_COEF_ref( j ),    &!in
           & temp0_k%profiles_COEF( i ),      &!inout
           & coef) !in 
    End Do
  End If 
 
  !----------------------------------------------------------------
  !12. K of profile variable interpolation - COEF levs to USER levs
  !----------------------------------------------------------------

! intvar logical array set by direct code
! loop over chanels inside subroutine
    If (addinterp) Then
    Call rttov_intavg_prof_k( &
 	    & nchannels,                       & !in 
            & nprofiles,                       & !in 
	    & lprofiles,                       & !in 
            & profiles(1)%nlevels,             & !in
            & temp0%profiles_COEF(1)%nlevels,  & !in
            & profiles,                        & !in     source variables
            & profiles_k_all,                  & !in     source variables
            & temp0%profiles_COEF,             & !inout  target variables
            & temp0_k%profiles_COEF,           & !inout  target variables
            & errorstatus)                       !out
      If ( Any(errorstatus == errorstatus_fatal) ) Then
        IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
        Return
      End If
    Else 
! COEF levs same as USER levs
     Do i = 1, nchannels
      profiles_k_all(i)%t(:)   =  profiles_k_all(i)%t(:) + temp0_k%profiles_COEF(i)%t(:) 
      profiles_k_all(i)%q(:)   =  profiles_k_all(i)%q(:) + temp0_k%profiles_COEF(i)%q(:)
      j = lprofiles(i)
      if(profiles(j) % clw_data)   profiles_k_all(i)%clw(:) =  profiles_k_all(i)%clw(:) + temp0_k%profiles_COEF(i)%clw(:)
      if(profiles(j) % ozone_data) profiles_k_all(i)%o3(:)  =  profiles_k_all(i)%o3(:)  + temp0_k%profiles_COEF(i)%o3(:)
      if(profiles(j) % co2_data)   profiles_k_all(i)%co2(:) =  profiles_k_all(i)%co2(:) + temp0_k%profiles_COEF(i)%co2(:)
      if(profiles(j) % n2o_data)   profiles_k_all(i)%n2o(:) =  profiles_k_all(i)%n2o(:) + temp0_k%profiles_COEF(i)%n2o(:)   
      if(profiles(j) % co_data)    profiles_k_all(i)%co(:)  =  profiles_k_all(i)%co(:)  + temp0_k%profiles_COEF(i)%co(:)    
      if(profiles(j) % ch4_data)   profiles_k_all(i)%ch4(:) =  profiles_k_all(i)%ch4(:) + temp0_k%profiles_COEF(i)%ch4(:)  
     Enddo
    EndIf

! complete profiles on COEF levs for surf, skin, cloud, aerosol, solar, angle
     Do i = 1, nchannels
      profiles_k_all(i) % ozone_Data = .False.  ! no meaning
      profiles_k_all(i) % co2_Data   = .False.  ! no meaning
      profiles_k_all(i) % n2o_Data   = .False.  ! no meaning
      profiles_k_all(i) % co_Data    = .False.  ! no meaning
      profiles_k_all(i) % ch4_Data   = .False.  ! no meaning
      profiles_k_all(i) % clw_Data   = .False.  ! no meaning

     profiles_k_all(i) % S2m % t      = profiles_k_all(i) % S2m % t + temp0_k%profiles_COEF(i) % S2m % t
     profiles_k_all(i) % S2m % q      = profiles_k_all(i) % S2m % q + temp0_k%profiles_COEF(i) % S2m % q
     profiles_k_all(i) % S2m % o      = profiles_k_all(i) % S2m % o + temp0_k%profiles_COEF(i) % S2m % o
     profiles_k_all(i) % S2m % p      = profiles_k_all(i) % S2m % p + temp0_k%profiles_COEF(i) % S2m % p
     profiles_k_all(i) % S2m % u      = profiles_k_all(i) % S2m % u + temp0_k%profiles_COEF(i) % S2m % u
     profiles_k_all(i) % S2m % v      = profiles_k_all(i) % S2m % v + temp0_k%profiles_COEF(i) % S2m % v
     profiles_k_all(i) % S2m % wfetc  = profiles_k_all(i) % S2m % wfetc + temp0_k%profiles_COEF(i) % S2m % wfetc

     profiles_k_all(i) % skin % surftype   = -1       ! no meaning
     profiles_k_all(i) % skin % watertype  = -1       ! no meaning
     profiles_k_all(i) % skin % t          = profiles_k_all(i) % skin % t         + temp0_k%profiles_COEF(i) % skin % t
     profiles_k_all(i) % skin % fastem(:)  = profiles_k_all(i) % skin % fastem(:) + temp0_k%profiles_COEF(i) % skin % fastem(:)

     profiles_k_all(i) % ctp        = profiles_k_all(i) % ctp       + temp0_k%profiles_COEF(i) % ctp 
     profiles_k_all(i) % cfraction  = profiles_k_all(i) % cfraction + temp0_k%profiles_COEF(i) % cfraction

!    profiles_k_all(i) % idg    = profiles_k_all(i) % idg + temp0_k%profiles_COEF(i) % idg
!    profiles_k_all(i) % ish    = profiles_k_all(i) % ish + temp0_k%profiles_COEF(i) % ish

    profiles_k_all(i) % zenangle   = profiles_k_all(i) % zenangle + temp0_k%profiles_COEF(i) % zenangle
    profiles_k_all(i) % azangle    = profiles_k_all(i) % azangle  + temp0_k%profiles_COEF(i) % azangle

    profiles_k_all(i) % sunzenangle = profiles_k_all(i) % sunzenangle + temp0_k%profiles_COEF(i) % sunzenangle
    profiles_k_all(i) % sunazangle  = profiles_k_all(i) % sunazangle  + temp0_k%profiles_COEF(i) % sunazangle
    profiles_k_all(i) % latitude    = profiles_k_all(i) % latitude    + temp0_k%profiles_COEF(i) % latitude
    profiles_k_all(i) % elevation   = profiles_k_all(i) % elevation   + temp0_k%profiles_COEF(i) % elevation
!    profiles_k_all(i) % addsolar    = profiles_k_all(i) % addsolar    + temp0_k%profiles_COEF(i) % addsolar
!    profiles_k_all(i) % addrefrac   = profiles_k_all(i) % addrefrac   + temp0_k%profiles_COEF(i) % addrefrac
enddo
!
  Do i = 1, nchannels
        profiles_k(i) % nlevels          = profiles_k_all(i) % nlevels
        profiles_k(i) % s2m % t          = profiles_k_all(i) % s2m % t
        profiles_k(i) % s2m % q          = profiles_k_all(i) % s2m % q
        profiles_k(i) % s2m % p          = profiles_k_all(i) % s2m % p
        profiles_k(i) % s2m % u          = profiles_k_all(i) % s2m % u
        profiles_k(i) % s2m % v          = profiles_k_all(i) % s2m % v
        profiles_k(i) % s2m % wfetc      = profiles_k_all(i) % s2m % wfetc 
        profiles_k(i) % skin % t         = profiles_k_all(i) % skin % t
        profiles_k(i) % skin % fastem(1) = profiles_k_all(i) % skin % fastem(1)
        profiles_k(i) % skin % fastem(2) = profiles_k_all(i) % skin % fastem(2)
        profiles_k(i) % skin % fastem(3) = profiles_k_all(i) % skin % fastem(3)
        profiles_k(i) % skin % fastem(4) = profiles_k_all(i) % skin % fastem(4)
        profiles_k(i) % skin % fastem(5) = profiles_k_all(i) % skin % fastem(5)
        profiles_k(i) % ctp              = profiles_k_all(i) % ctp
        profiles_k(i) % cfraction        = profiles_k_all(i) % cfraction

        Do  ii=1,profiles(1)%nlevels
           profiles_k(i) % p(ii)         = profiles_k_all(i) % p(ii)
           profiles_k(i) % t(ii)         = profiles_k_all(i) % t(ii)
           profiles_k(i) % q(ii)         = profiles_k_all(i) % q(ii)
           if(profiles(j) % ozone_data) profiles_k(i) % o3(ii)  = profiles_k_all(i) % o3(ii)
           if(profiles(j) % co2_data)   profiles_k(i) % co2(ii) = profiles_k_all(i) % co2(ii)
           if(profiles(j) % n2o_data)   profiles_k(i) % n2o(ii) = profiles_k_all(i) % n2o(ii)
           if(profiles(j) % co_data)    profiles_k(i) % co (ii) = profiles_k_all(i) % co(ii)
           if(profiles(j) % ch4_data)   profiles_k(i) % ch4(ii) = profiles_k_all(i) % ch4(ii)
           if(profiles(j) % clw_data)   profiles_k(i) % clw(ii) = profiles_k_all(i) % clw(ii)
           if(addaerosl)then
             profiles_k(i) % aerosols(1:11,ii)= profiles_k_all(i) % aerosols(1:11,ii) 
           endif
           if(addclouds)then
             profiles_k(i) % cloud  (1:6,ii)= profiles_k_all(i) % cloud  (1:6,ii)
             profiles_k(i) % cfrac  (1:6,ii)= profiles_k_all(i) % cfrac  (1:6,ii)
           endif
        End Do
   End Do

  !-----------------------------------------
  !13. deallocate memory for local variables
  !-----------------------------------------


  if(addaerosl)then
    Do i=1,nchannels

      DeAllocate( aux_prof_k(i)%relhum   ,stat= alloc_status(7))

    If( Any(alloc_status /= 0) ) Then
      errorstatus(:) = errorstatus_fatal
      Write( errMessage, '( "deallocation of aux_prof" )' )
      Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
      Return
    endif

    enddo

    Do i=1,nprofiles


      DeAllocate( aux_prof(i)%iaernum  ,stat= alloc_status(7))
      DeAllocate( aux_prof(i)%iaertyp  ,stat= alloc_status(8))
      DeAllocate( aux_prof(i)%relhum   ,stat= alloc_status(9))
      DeAllocate( aux_prof(i)%relhumref,stat= alloc_status(10))

    If( Any(alloc_status /= 0) ) Then
      errorstatus(:) = errorstatus_fatal
      Write( errMessage, '( "deallocation of aux_prof" )' )
      Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
      Return
    endif

    enddo
  endif

  if(addaerosl.or.addclouds)then


    Call rttov_alloc_trans_scatt_ir( alloc_status(1), transmission_scatt_ir_k, &
      nchannels, profiles(1) %nlevels, 0_jpim )
    Call rttov_alloc_trans_scatt_ir( alloc_status(2), transmission_scatt_ir_stream_k, &
      nchannels, profiles(1) %nlevels, 0_jpim  )
    Call rttov_alloc_trans_scatt_ir( alloc_status(3), transmission_scatt_ir, &
      nchannels, profiles(1) %nlevels, 0_jpim )
    Call rttov_alloc_trans_scatt_ir( alloc_status(4), transmission_scatt_ir_stream, &
      nchannels, profiles(1) %nlevels, 0_jpim )


    If( Any(alloc_status /= 0) ) Then
      errorstatus(:) = errorstatus_fatal
      Write( errMessage, '( "deallocation of trans_scatt_ir" )' )
      Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
      Return
    endif

  endif





  Call rttov_alloc_transmission( alloc_status(1), transmission_k, profiles(1) % nlevels, nchannels, &
  & 0_jpim, nstreams = nstreams, calc = .true. )



    If( Any(alloc_status /= 0) ) Then
      errorstatus(:) = errorstatus_fatal
      Write( errMessage, '( "deallocation of transmission_k" )' )
      Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
      Return
    endif


  Call rttov_alloc_transmission( alloc_status(1), transmission, profiles(1) % nlevels, nchannels, &
  & 0_jpim, nstreams = nstreams, calc = .true. )


    If( Any(alloc_status /= 0) ) Then
      errorstatus(:) = errorstatus_fatal
      Write( errMessage, '( "deallocation of transmission" )' )
      Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
      Return
    endif

    DeAllocate(auxrad_stream_k % up     ,stat= alloc_status(1))
    DeAllocate(auxrad_stream_k % down   ,stat= alloc_status(2))
    DeAllocate(auxrad_stream_k % cloudy ,stat= alloc_status(12))
    DeAllocate(auxrad_stream % rad_tmp          ,stat= alloc_status(1))
    DeAllocate(auxrad_stream % meanrad_up       ,stat= alloc_status(2))
    DeAllocate(auxrad_stream % meanrad_up_2d    ,stat= alloc_status(3))
    DeAllocate(auxrad_stream % meanrad_down     ,stat= alloc_status(4))
    DeAllocate(auxrad_stream % meanrad_down_ref ,stat= alloc_status(5))

    If( Any(alloc_status /= 0) ) Then
      errorstatus(:) = errorstatus_fatal
      Write( errMessage, '( "deallocation of auxrad_stream_k" )' )
      Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
      Return
    endif

    if(addsun)then
      if(addaerosl.or.addclouds)then
        DeAllocate(auxrad_stream_k % FAC1_1    ,stat= alloc_status(1))
        DeAllocate(auxrad_stream_k % FAC2_1    ,stat= alloc_status(2))
        DeAllocate(auxrad_stream_k % FAC3_1    ,stat= alloc_status(3))
        DeAllocate(auxrad_stream_k % FAC4_1    ,stat= alloc_status(4))
        DeAllocate(auxrad_stream_k % FAC5_1    ,stat= alloc_status(5))
        DeAllocate(auxrad_stream_k % FAC6_1    ,stat= alloc_status(6))
        DeAllocate(auxrad_stream_k % FAC7_1    ,stat= alloc_status(7))
        DeAllocate(auxrad_stream_k % FAC1_2    ,stat= alloc_status(8))
        DeAllocate(auxrad_stream_k % FAC2_2    ,stat= alloc_status(9))
        DeAllocate(auxrad_stream_k % FAC3_2    ,stat= alloc_status(10))
        DeAllocate(auxrad_stream_k % FAC4_2    ,stat= alloc_status(11))
        DeAllocate(auxrad_stream_k % FAC5_2    ,stat= alloc_status(12))
        DeAllocate(auxrad_stream_k % FAC6_2    ,stat= alloc_status(13))
        DeAllocate(auxrad_stream_k % FAC7_2    ,stat= alloc_status(14))
        DeAllocate(auxrad_stream_k % FAC1_3    ,stat= alloc_status(15))
        DeAllocate(auxrad_stream_k % FAC2_3    ,stat= alloc_status(16))
        DeAllocate(auxrad_stream_k % FAC3_3    ,stat= alloc_status(17))
        DeAllocate(auxrad_stream_k % FAC4_3    ,stat= alloc_status(18))
        DeAllocate(auxrad_stream_k % FAC5_3    ,stat= alloc_status(19))
        DeAllocate(auxrad_stream_k % FAC6_3    ,stat= alloc_status(20))
        DeAllocate(auxrad_stream_k % FAC7_3    ,stat= alloc_status(21))
    If( Any(alloc_status /= 0) ) Then
      errorstatus(:) = errorstatus_fatal
      Write( errMessage, '( "deallocation of auxrad_stream_k" )' )
      Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
      Return
    endif

      endif
    endif
    DeAllocate(auxrad_stream % up     ,stat= alloc_status(1))
    DeAllocate(auxrad_stream % down   ,stat= alloc_status(2))
    DeAllocate(auxrad_stream % down_ref   ,stat= alloc_status(3))
    DeAllocate(auxrad_stream % cloudy ,stat= alloc_status(13))

    If( Any(alloc_status /= 0) ) Then
      errorstatus(:) = errorstatus_fatal
      Write( errMessage, '( "deallocation of auxrad_stream" )' )
      Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
      Return
    endif

     if(addsun)then
      if(addaerosl.or.addclouds)then
        DeAllocate(auxrad_stream % FAC1_1    ,stat= alloc_status(1))
        DeAllocate(auxrad_stream % FAC2_1    ,stat= alloc_status(2))
        DeAllocate(auxrad_stream % FAC3_1    ,stat= alloc_status(3))
        DeAllocate(auxrad_stream % FAC4_1    ,stat= alloc_status(4))
        DeAllocate(auxrad_stream % FAC5_1    ,stat= alloc_status(5))
        DeAllocate(auxrad_stream % FAC6_1    ,stat= alloc_status(6))
        DeAllocate(auxrad_stream % FAC7_1    ,stat= alloc_status(7))
        DeAllocate(auxrad_stream % FAC1_2    ,stat= alloc_status(8))
        DeAllocate(auxrad_stream % FAC2_2    ,stat= alloc_status(9))
        DeAllocate(auxrad_stream % FAC3_2    ,stat= alloc_status(10))
        DeAllocate(auxrad_stream % FAC4_2    ,stat= alloc_status(11))
        DeAllocate(auxrad_stream % FAC5_2    ,stat= alloc_status(12))
        DeAllocate(auxrad_stream % FAC6_2    ,stat= alloc_status(13))
        DeAllocate(auxrad_stream % FAC7_2    ,stat= alloc_status(14))
        DeAllocate(auxrad_stream % FAC1_3    ,stat= alloc_status(15))
        DeAllocate(auxrad_stream % FAC2_3    ,stat= alloc_status(16))
        DeAllocate(auxrad_stream % FAC3_3    ,stat= alloc_status(17))
        DeAllocate(auxrad_stream % FAC4_3    ,stat= alloc_status(18))
        DeAllocate(auxrad_stream % FAC5_3    ,stat= alloc_status(19))
        DeAllocate(auxrad_stream % FAC6_3    ,stat= alloc_status(20))
        DeAllocate(auxrad_stream % FAC7_3    ,stat= alloc_status(21))
    If( Any(alloc_status /= 0) ) Then
      errorstatus(:) = errorstatus_fatal
      Write( errMessage, '( "deallocation of auxrad_stream" )' )
      Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
      Return
    endif
      endif
    endif


  do i=1,nprofiles
    If ( coef % id_sensor == sensor_id_hi ) Then
      If (coef%fmv_model_ver == 9) Then
        Deallocate ( sunglint  (i) % beta     ,stat= alloc_status(17))
        Deallocate ( sunglint  (i) % psi      ,stat= alloc_status(18))
      End if
    End if

     If( Any(alloc_status /= 0) ) Then
       errorstatus(:) = errorstatus_fatal
       Write( errMessage, '( "deallocation of raytracing")' )
       Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
       IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
       Return
     End If
  enddo









  Do i = 1, nchannels
    If ( coef % id_sensor == sensor_id_hi ) Then
      If (coef%fmv_model_ver == 9) Then
        Deallocate ( sunglint_k  (i) % beta     ,stat= alloc_status(17))
        Deallocate ( sunglint_k  (i) % psi      ,stat= alloc_status(18))
      End if
    End if
     If( Any(alloc_status /= 0) ) Then
       errorstatus(:) = errorstatus_fatal
       Write( errMessage, '( "deallocation of k of raytracing")' )
       Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
       IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
       Return
     End If
  enddo

 
 Deallocate(auxrad % layer   ,stat= alloc_status(1))
  Deallocate(auxrad % surfair ,stat= alloc_status(2))
  Deallocate(auxrad % skin    ,stat= alloc_status(3))
  Deallocate(auxrad % cosmic  ,stat= alloc_status(4))
! If ( addcloud ) Then
!    Deallocate(auxrad % down_cloud ,stat= alloc_status(16))
! End If
  If( Any(alloc_status /= 0) ) Then
     errorstatus(:) = errorstatus_fatal
     Write( errMessage, '( "deallocation of aux radiances")' )
     Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
     IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
     Return
  End If

  Do i = 1, nprofiles
     If( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po ) Then
        If( Associated( aux_prof(i) % debye_prof) ) Then
           Deallocate( aux_prof(i) % debye_prof ,stat= alloc_status(1))
           If( Any(alloc_status /= 0) ) Then
              errorstatus(:) = errorstatus_fatal
              Write( errMessage, '( "deallocation of debye profile")' )
              Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
           End If
        End If
     Endif
  End Do
  Do i = 1, nchannels
     If( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po ) Then
        If( Associated( aux_prof_k(i) % debye_prof) ) Then
           Deallocate( aux_prof_k(i) % debye_prof ,stat= alloc_status(1))
           If( Any(alloc_status /= 0) ) Then
              errorstatus(:) = errorstatus_fatal
              Write( errMessage, '( "deallocation of K debye profile")' )
              Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
           End If
        End If
     Endif
  End Do

  if(addclouds)then
    Do i = 1, nprofiles
      Deallocate( aux_prof(i) % dg ,stat= alloc_status(1))
      Deallocate( aux_prof(i) % fac1_dg ,stat= alloc_status(2))
      Deallocate( aux_prof(i) % fac2_dg ,stat= alloc_status(3))
      Deallocate( aux_prof(i) % fac3_dg ,stat= alloc_status(4))
           If( Any(alloc_status /= 0) ) Then
              errorstatus(:) = errorstatus_fatal
              Write( errMessage, '( "deallocation of dg profile")' )
              Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
           End If
    enddo
  endif



  if(addclouds)then
    Do i = 1, nchannels
      Deallocate( aux_prof_k(i) % dg ,stat= alloc_status(1))
      Deallocate( aux_prof_k(i) % fac1_dg ,stat= alloc_status(2))
      Deallocate( aux_prof_k(i) % fac2_dg ,stat= alloc_status(3))
      Deallocate( aux_prof_k(i) % fac3_dg ,stat= alloc_status(4))
           If( Any(alloc_status /= 0) ) Then
              errorstatus(:) = errorstatus_fatal
              Write( errMessage, '( "deallocation of dg profile")' )
              Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
           End If
    enddo
  endif


  Call rttov_alloc_prof( alloc_status(1), nchannels, profiles_k_all, profiles(1)%nlevels, coef_scatt_ir, 0_jpim, &
    & addaerosl = addaerosl, addclouds = addclouds, blob = profiles_k_all_blob )

  If( Any(alloc_status /= 0) ) Then
     errorstatus(:) = errorstatus_fatal
     Write( errMessage, '( "deallocation of profiles_COEF_k")' )
     Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
     IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
     Return
  End If

  If( local_rad_k ) Then
     Call rttov_alloc_rad( alloc_status(1), nchannels, radiancedata_k, profiles(1)%nlevels, 0_jpim )
     If( Any(alloc_status /= 0) ) Then
        errorstatus(:) = errorstatus_fatal
        Write( errMessage, '( "deallocation of K radiances")' )
        Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
        IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)
        Return
     End If
  End If


  If ( apply_reg_limits ) Then
    Call rttov_alloc_prof( alloc_status(2), nprofiles, profiles_COEF_ref, &
      coef%nlevels, coef_scatt_ir, 0_jpim, blob = profiles_COEF_blob_ref )
  End If 
  If( any(alloc_status /= 0) ) then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "deallocation of profiles(i)_COEF")' )
     Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
     Return
  End If

  Do i = 1, nprofiles
     If( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po ) Then
        If( Associated( aux_prof_COEF(i) % debye_prof) ) Then
           Deallocate( aux_prof_COEF(i) % debye_prof ,stat= alloc_status(1))
           If( Any(alloc_status /= 0) ) Then
              errorstatus(:) = errorstatus_fatal
              Write( errMessage, '( "deallocation of  debye profile (COEF)")' )
              Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
           End If
        End If
     Endif
  End Do
  Do i = 1, nchannels
     If( coef % id_sensor == sensor_id_mw .or. coef % id_sensor == sensor_id_po ) Then
        If( Associated( aux_prof_COEF_k(i) % debye_prof) ) Then
           Deallocate( aux_prof_COEF_k(i) % debye_prof ,stat= alloc_status(1))
           If( Any(alloc_status /= 0) ) Then
              errorstatus(:) = errorstatus_fatal
              Write( errMessage, '( "deallocation of K debye profile (COEF)")' )
              Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
           End If
        End If
     Endif
  End Do





  Call rttov_check_temp( &
    & temp_status,                             &
    & nprofiles,                               &
    & nchannels,                               &
    & channels,                                &
    & addinterp,                               &
    & profiles(1)%nlevels,                     &
    & coef,                                    &
    & coef_scatt_ir,                           &
    & optp,                                    &
    & addsun .or. Any( profiles(:)%addsolar ), &
    & addaerosl,                               &
    & addclouds,                               &
    & 0_jpim,                                  &
    & temp0 = temp0, temp0_k = temp0_k,        &   
    & temp1 = temp1, temp1_k = temp1_k,        &   
    & temp2 = temp,  temp2_k = temp_k )  
  If( temp_status .ne. errorstatus_success ) Then
    Write( errMessage, '( "rttov_check_temp fatal error ")' )
    Call Rttov_ErrorReport( errorstatus_fatal, errMessage, NameOfRoutine )
    Return
  EndIf
  


  IF (LHOOK) CALL DR_HOOK('RTTOV_K',1_jpim,ZHOOK_HANDLE)



End Subroutine rttov_k
